// ****************************************************************************
//
// Routine:      acReadTARGET
// Author:       T.Oest
//
// Description:  copy the target position to GTAR and
//               apply alignment corrections 
//               this was done before in rc99ATARGET
//
//           
//
// Modification:
// EVHD filled now in acread                                        08/03/00 TO
// fill DTA1 bank                                                   10/03/00 TO
// corrected include path bug                                       16/08/00 AG      
//
// ****************************************************************************
// ****************************************************************************
//
//  theses are the comments to the old routine rc99ATARGET
//
//  Routine:      rc99ATARGET
//  Author:       T.Oest/D.Samtleben
//
//  Description:  target rates, position
//
//  1998 data: 8 wires
//             1. GTAR table row  Ia    Al
//             2.            row  Ib    Ti
//             3.            row  Ii    Al
//             4.            row  Io    Al
//             5.            row IIa    Fe  (45 degree)
//             6.            row IIb    Al  (45 degree)
//             7.            row IIi    C
//             8.            row IIo    Ti
//
//
//  Modification:
//  -  dropping of GTAR is now included in rc99AStandAlone
//                                             01/07/99   Samtleben
//  -  date and time is only filled if DDAQ exist
//                                             08/08/99   T.Oest
//  -  return statement included to avoid warnings
//                                             24/08/99   T.Oest
//
// ****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

// ONLINE includes
#include "esp.h"

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"

// globals
static int Nrfile  = 0;
static char * DBfile[100];

// prototypes
int acfillGTARfromDTAR();

//*****************************************************************
// function
//*****************************************************************

int
acReadTARGET( int KTstatus )
{
  ARTE_mark( "acReadTARGET", 0 );

//==================================================================
// no action for MC
//==================================================================

  if (ArteTable<MTRA>::size() > 0 ) return 0;

//===================================================================
// remove the GTAR for real data
//===================================================================

  ArteTable<GTAR>::clear();

//==================================================================
// 1. copy from DTA1 to GTAR 
//==================================================================

  if (ArteTable<DTA1>::size() > 0){


   for( ArteTable<DTA1>::iterator dta1 = ArteTable<DTA1>::begin();
        dta1 != ArteTable<DTA1>::end(); ++dta1 ) {


         GTAR gtar;

         gtar.cmp    = (*dta1)->cmp;
         gtar.wnr    = (*dta1)->wnr;
         gtar.dx     = (*dta1)->dx;
         gtar.dy     = (*dta1)->dy;
         gtar.dz     = (*dta1)->dz;
         gtar.x      = (*dta1)->x;
         gtar.y      = (*dta1)->y;
         gtar.z      = (*dta1)->z;
         gtar.ms     = (*dta1)->ms;
         gtar.x0bm   = (*dta1)->x0bm;
         gtar.y0bm   = (*dta1)->y0bm;
         gtar.sxbm   = (*dta1)->sxbm;
         gtar.sybm   = (*dta1)->sybm;
         gtar.shap   = (*dta1)->shap;
         gtar.name   = (*dta1)->name;
         gtar.eff    = (*dta1)->eff;

         for (int i = 0; i<=8 ; i++) gtar.rot[i]    = (*dta1)->rot[i];

         ArteTable<GTAR>::push_back( gtar );
     };

  } else { 

//==================================================================
// 2. copy from DTAR to GTAR/DTA1  (database access switched off)
//==================================================================


  if (ArtePointer<EVHD>(1)->date > 990200) {

   int flag = acfillGTARfromDTAR();

   DTA1 dta1;

     for( ArteTable<GTAR>::iterator tar = ArteTable<GTAR>::begin();
       tar != ArteTable<GTAR>::end(); ++tar ) {


         dta1.cmp    = (*tar)->cmp;
         dta1.wnr    = (*tar)->wnr;
         dta1.dx     = (*tar)->dx;
         dta1.dy     = (*tar)->dy;
         dta1.dz     = (*tar)->dz;
         dta1.x      = (*tar)->x;
         dta1.y      = (*tar)->y;
         dta1.z      = (*tar)->z;
         dta1.ms     = (*tar)->ms;
         dta1.x0bm   = (*tar)->x0bm;
         dta1.y0bm   = (*tar)->y0bm;
         dta1.sxbm   = (*tar)->sxbm;
         dta1.sybm   = (*tar)->sybm;
         dta1.shap   = (*tar)->shap;
         dta1.name   = (*tar)->name;
         dta1.eff    = (*tar)->eff;

         for (int i=0; i<=8 ; i++) dta1.rot[i]    = (*tar)->rot[i];

         ArteTable<DTA1>::push_back( dta1 );

     };


     return flag;



  };

  return -1;

  };
 return 0;
}
/************************************************************************
* KATRGT  target database files
*
* called from  KUIP
***********************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
//#include "arte/Arte.hh"

extern "C" void kugetc_( char*, int *, int );


void katrgt_() {
  //extern "C" void katrgt_() {

//---------------------------------------------------------------
//   read in subdetector
//---------------------------------------------------------------
  if (Nrfile < 100){

   int len;
   int len1 = 100;
   char input[120];
   kugetc_(input, &len, len1);
   for (int i =0; i<len; i++) {input[i] = tolower(input[i]);};
   input[len] = 0;

   DBfile[Nrfile]  = (char *) malloc(strlen(input)+1);
   sprintf(DBfile[Nrfile],"%s", input);

   Nrfile++;

  } else {
    cout << " too many target database files " << endl;
  };

};
