//                              -*- Mode: C++ -*-
//: ArteMessage.h
//
// 
//
// Author: Mauro Villa
// ECAL group
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/5/97
// Last Update ........
//
#ifndef ARTEMESSAGE_HH_
#define ARTEMESSAGE_HH_
#include "arte/MEPack.hh"
#include <string>
#include <iostream>
#include <strstream>

/*
CLASS
  ArteMessage
  Interface class to print messages, warning or error informations
*/

struct ArteBuffer { 
  //  The stream buffer to store temporary data
  enum {SIZEBUF=200};
  char  msg[SIZEBUF];
  inline int size() const { return SIZEBUF;};
  inline char* string() { return msg;};
  inline ArteBuffer() {
    msg[0] = 0;
  }
};

const unsigned int maxwarn = 20;

class ArteMessage : public ArteBuffer, public std::ostrstream {
public:
  //-- 
  // Constructor: it does nothing.
  ArteMessage();
  //--
  //  Destructor: it does nothing for this class. 
  ~ArteMessage();
  //--
  // Write the stored message on all printers
  ArteMessage& allp();
  //--
  // count and print a message 
  ArteMessage& allp(int & ID, int NP);

  //--
  // print text on all printers declared by CTPL 
  ArteMessage& toPrinters(char* CTPL);

  //--
  // Write error message on all printer units.
  //      Batch: stop program.
  //      Interactive: stop or continue. 
  ArteMessage& error();
  //--
  // error message; count for final statistics
  ArteMessage& error(int &ID);

  //--
  //  Print a warning
  ArteMessage& warn();
  //--
  //  Print and Count a warning
  ArteMessage& warn(int &ID);
  //--
  //  Print and Count a warning
  ArteMessage& warn(int &ID, int ncount);

  //--
  // Write Message on all printers
  inline void toAllPrinters(char* CTXT);
  //--
  // error message; count for final statistics
  inline void errorToCount(int &ID, char* TEXT);
  //--
  // Write error message on all printer units.
  //      Batch: stop program.
  //      Interactive: stop or continue. 
  inline void error(char* CTXT);
  // --
  //  list, count, handle an error 
  //void memesc_(int* ID, int* NP, int * NF, char* TEXT, int len);
  //--
  // count and print a message 
  inline void printAndCount(int & ID, int NP, char* TEXT);
  //--
  // print text CTXT on all printers declared by CTPL 
  inline void toSelectedPrinters(char* TEXT, char* CTPL);
  //--
  // warning; count number of occurrencies 
  inline void warningToCount(int &ID, char* TEXT);
  //--
  // print warning 
  inline void warning(char* TEXT);

private:
  //--
  //  Clear buffer
  void clear();
  //-- 
};

extern ArteMessage arteMsg;

#include "arte/ArteMessage.icc"
#endif // ARTEMESSAGE_HH_
