//                              -*- Mode: C++ -*-
//: CmdCareCalib.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
/* $Log: CmdCareCalib.C,v $
 * Revision 1.5  2000/01/11 11:24:31  natasha
 * care (3.50):
 * The major changes with respect to the previous release are:
 * 1) Digi MC inizialization is skipped for real data.
 * 2) the care/dglev command has been removed in favour of the hbdigi/dglev
 * 3) MC calibration constants retuned on MC data with magnet ON
 * 4) Cluster z correction retuned
 * 5) More flexibility for cluster threshold settings (ECCutFunction.C)
 * 6) The most important warning messages are counted
 * 7) No more chashes on missing DB
 * 8) Full C++ 9X standard conformance of the code
 *
 * Revision 1.4  1999/08/04 15:21:28  natasha
 * new care version from Mauro
 *
 * Revision 1.2  1999/06/01  16:07:30  villa
 * before HH
 *
 */
#include "care/CmdCareCalib.hh"
#include "care/ECMCcalib.hh"


/////////////// The function seen by kuip
#ifdef USEKUIP
extern "C" {
  void carecalib_(){
    CmdCareCalib calCmd;
    calCmd.readAndExe();
  }
}; // end of extern
#endif
//////////////////////////////////////////////////////////

/*
CLASS
  CmdCareCalib
  Class to deal with the care/calib kuip command.

*/

const float cfinner = 28.17;
const float cfmiddle = 6.59;
const float cfouter = 6.34;

CmdCareCalib::CmdCareCalib() : inner(cfinner), middle(cfmiddle),
		outer(cfouter) {}

CmdCareCalib::~CmdCareCalib(){}

void CmdCareCalib::defValues(){
  inner = cfinner;
  middle = cfmiddle;
  outer = cfouter;
}

//--
//   Execute stored command
void CmdCareCalib::execute(){
  ECMCcalib::setCalibration(inner,middle,outer);
}

# ifdef USEKUIP
//--
//   Read a command from external
void CmdCareCalib::read(){
  inner = readFloat();
  middle = readFloat();
  outer = readFloat();
}
#else
//void CmdCareCalib::read(){}
# endif                          
//--
//   Printout Information for the current MinimumEnergy data stored
void CmdCareCalib::dump(){
  ECMCcalib::dumpCalib();
}






