//                              -*- Mode: C++ -*-
//: CmdCareCalerr.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#ifndef CMDCARECALERR_HH_
#define CMDCARECALERR_HH_

#include "care/CmdBase.hh" 


/**
class  CmdCareCalerr.
  Class to deal with the care/calerr kuip command.
  It is used to set the precision of ECAL calibration.

  Kuip syntax:  CARE/CALERR  PREC

  where PREC is the desired precision on calibration.
  Example: CARE/CALERR 0.05 means to have a 5 % precision.
*/

class CmdCareCalerr : public CmdBase {
public:
  /// Constructor
  CmdCareCalerr();
  /// Constructor with precision set
  inline CmdCareCalerr(float prec);
  //-- Destructor
  /// Destructor: currently void
  virtual ~CmdCareCalerr();
  
  //-- Behavioural operations
  ///   Reset stored data to default values
  virtual void defValues();
  //--
  ///   Execute stored command
  virtual void execute();

# ifdef USEKUIP
  //--
  ///   Read a command from external
  virtual void read();
# endif
  //--
  ///    Dump internal data
  virtual void dump();
  //--
  ///   Return the stored precision on calibration
  static inline float precfactor();
  //--
  /**   Return a random factor with mean 1 and
        sigma given by the precision on calibration */
  float factor();

private:
  //--
  static  float precision;
};


// INLINES for  CmdCareCalerr

//   Return the stored precision on calibration
float CmdCareCalerr::precfactor() {
  return precision;
}

// Constructor with precision set
CmdCareCalerr::CmdCareCalerr(float prec){
  precision=prec;
};

#endif  //   CMDCARECALERR_HH_

