//                              -*- Mode: C++ -*-
//: EAClusTrackInfo.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 2/5/97
// Last Update ........
//

#ifndef EACLUSTRACKANALYZER_HH_
#define EACLUSTRACKANALYZER_HH_

#include "care/EAClusterAnalyzer.hh"
#include "care/ECLastHit.hh"

/**
class EAClusTrackInfo.
  Class for finding informations on stored tracks.

*/

class EAClusTrackInfo : public EAClusterAnalyzer {
public:
  //-- Constructors
  /// Only default constructor is provided
  EAClusTrackInfo(const EMCluster* clus);
  /// The virtual default destructor
  virtual ~EAClusTrackInfo();
  //-- 
  // ClusterMember Analyzer
  //
  /**@name Beheavioral methods */
  //@{ 
  /// EMClusterMember Analyzer.
  virtual void doAnalysis(const EMClusterMember * clsMem);
  //--
  /// EMHitCell Analyzer.
  virtual void doAnalysis(const EMHitCell * hitT) ;
  //--
  /// EMCellFraction Analyzer.
  inline
  virtual void doAnalysis(const EMCellFraction * hitT) ;
  //--
  /// EMCluster Analyzer.
  virtual void doAnalysis(const EMCluster * clus);
  //--
  /// EMClusterTree Analyzer.
  virtual void doAnalysis(const EMClusterTree * tree);
  //--
  /// EMTrackOnEcal Analyzer.
  virtual void doAnalysis(const EMTrackOnEcal * track);
  //--
  ///  Base function to be called with the TopCluster List.
  void analyzeTopList(const EMClusterTree & topList);
  //@}
  //--
  ///  return track informations stored.
  TrackInfo getTrack() const;
 
private:
  //--
  ///  std::vector containing the cluster pointers
  const EMTrackOnEcal * track;
};

void EAClusTrackInfo::doAnalysis(const EMCellFraction * hitT) {
  doAnalysis((const EMHitCell *) hitT);
}

#endif // EACLUSTRACKANALYZER_HH_
