
#include <time.h>

#include <string>
#include <algorithm>

#include <Gate/ObjArray.hh>
#include <Gate/ClonesArray.h>
#include <arte/RSEG_cmp.hh>

#include <Clue/RtEvent.h>
#include <Clue/Wire.hh>
#include <Clue/MC_Track.hh>
#include <Clue/MC_Vertex.hh>
#include <Clue/RecoVertex.hh>
#include <Clue/RecoTrack.hh>
#include <Clue/TrackClones.hh>
#include <Clue/CustomTrack.hh>
#include <Clue/Cluster.hh>
#include <Clue/vtstruct.hh>
#include <Clue/vtproto.hh>
#include <Clue/ClTOPP.hh>

#include <Clue/UtaConversions.h>


int RtEvent::init( int )
{
  Clear();

  // if ClTOPP is not initialized yet, do it now
  if( toppLink==0 ) toppLink = new GClonesArray<ClTOPP*>( 100 );
  if( toppLink->size() != ArteTable<TOPP>::size() ) {
#ifdef USE_ROOT
    toppLink->Delete();
#else
#warning missing code (C.B. 8/31/99)
#endif
    for( ArteTable<TOPP>::iterator topp = ArteTable<TOPP>::begin();
	 topp != ArteTable<TOPP>::end();
	 ++topp ) {
      ClTOPP temp( *topp );
      toppLink->push_back( &temp );
    }
  }
  
  if( ArteTable<EVHD>::size()==1 ) {
    evt     = (*ArteTable<EVHD>::begin())->evt;
    run     = (*ArteTable<EVHD>::begin())->run;
    exp     = (*ArteTable<EVHD>::begin())->exp;
    vfld    = (*ArteTable<EVHD>::begin())->vfld;
    flagdst = (*ArteTable<EVHD>::begin())->flagdst;
/*
    dbmajor = (*ArteTable<EVHD>::begin())->dbmajor;
    dbminor = (*ArteTable<EVHD>::begin())->dbminor;
*/
    flag    = (*ArteTable<EVHD>::begin())->flag;

    int date    = (*ArteTable<EVHD>::begin())->date;
    int time    = (*ArteTable<EVHD>::begin())->time;
    
    struct tm tmp;
    tmp.tm_sec   = time % 100;
    tmp.tm_min   = (time % 10000) / 100;
    tmp.tm_hour  = time / 10000;
    tmp.tm_mday  = date % 100;
    tmp.tm_mon   = (date % 10000) / 100;
    tmp.tm_year  = date / 10000;
    tmp.tm_isdst = -1;
    
    unixtime = mktime( &tmp );
  } else {
    evt = run = exp = 0;
    vfld = 0;
    flagdst = dbmajor = dbminor = flag = unixtime = 0;
  }
  
  if( ArteTable<DIHD>::size()==1 ) {
    bx      = (*ArteTable<DIHD>::begin())->bx;
  } else bx = 0;
  
  for( ArteTableIterator<GTAR> gtar = ArteTable<GTAR>::begin();
       gtar != ArteTable<GTAR>::end();
       ++gtar ) {
    ClWire w( *gtar );
    wires->push_back( &w );
  }

  //
  // matched and reconstructed tracks
  //
  for( ArteTableIterator<RTRA> rtra = ArteTable<RTRA>::begin();
       rtra != ArteTable<RTRA>::end();
       ++rtra ) {
    RecoTrack r( *rtra );
    tracks->push_back( &r );
  }

  //
  // find tracks sharing RCCLs (gives a TrackClones entry)
  //
  for( ArteTable<RCCL>::iterator rccl = ArteTable<RCCL>::begin();
       rccl != ArteTable<RCCL>::end(); ++rccl ) {

    if( distance( (**rccl).rtra.begin(), (**rccl).rtra.end() )>1 ) {

      ArtePointer<TrackClones> clones
	= ArteTable<TrackClones>::push_back( TrackClones( Rsegc::ecal ));

      for( ArteRelationIterator<RTRA> rtra = (**rccl).rtra.begin();
	   rtra != (**rccl).rtra.end();
	   ++rtra ) {
	ArtePointer<RecoTrack> rTrack
	  = *( ArteTable<RecoTrack>::begin()
	       + ( ArteTable<RTRA>::iterator(*rtra)
		   - ArteTable<RTRA>::begin() ) );
	createRelation( clones, rTrack, &TrackClones::tracks );
      }
    }
  }

  //
  // find tracks sharing segments (gives a TrackClones entry)
  //
  for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
       rseg != ArteTable<RSEG>::end(); ++rseg ) {

    if( distance( (**rseg).rtra.begin(), (**rseg).rtra.end() )>1 ) {

      ArtePointer<TrackClones> clones
	= ArteTable<TrackClones>::push_back
	( TrackClones( (**rseg).cmp ));
      
      for( ArteRelationIterator<RTRA> rtra = (**rseg).rtra.begin();
	   rtra != (**rseg).rtra.end();
	   ++rtra ) {
	ArtePointer<RecoTrack> rTrack
	  = *( ArteTable<RecoTrack>::begin()
	       + ( ArteTable<RTRA>::iterator(*rtra)
		   - ArteTable<RTRA>::begin() ) );
	createRelation( clones, rTrack, &TrackClones::tracks );
      }
    }
  }

  if( storeRSEG_Flag ) {
    //
    for( ArteTableIterator<RSEG> rseg = ArteTable<RSEG>::begin();
	 rseg != ArteTable<RSEG>::end();
	 ++rseg ) {
      if( (*rseg)->cmp & Rsegc::vxd ) {
	VDS_Track v( *rseg );
	vds->push_back( &v );

      } else if( (*rseg)->cmp & Rsegc::patt && (*rseg)->fit==5 ) {
	OTR_SpacePoint o( *rseg );
	otr->push_back( &o );

      } else if( (*rseg)->cmp & Rsegc::rich ) {
	RICH_Track r( *rseg );
	rich->push_back( &r );
      
      } else if( (*rseg)->cmp & Rsegc::muon ) {
	MuonTrack m( *rseg );
	muon->push_back( &m );
      }
    }

    //
    // clusters
    //
    for( ArteTable<RCCL>::iterator rccl = ArteTable<RCCL>::begin();
	 rccl != ArteTable<RCCL>::end(); ++rccl ) {
      EcalCluster e( *rccl );
      clusters->push_back( &e );
    }
  }

  //
  // reconstructed vertices
  //
  for( ArteTable<RVER>::iterator rver = ArteTable<RVER>::begin();
       rver != ArteTable<RVER>::end(); ++rver ) {
    RecoVertex v( *rver );
    vertices->push_back( &v );
    
    ArtePointer<RecoVertex> rVertex = ArteTable<RecoVertex>::begin()
      + ( rver - ArteTable<RVER>::begin() );
    double chi2 = 0;
    
    for( ArteRelationIterator<RREL> rrel = (*rver)->rrel.begin();
	 rrel != (*rver)->rrel.end();
	 ++rrel ) {
#ifdef USE_ROOT
      if( (*rrel)->rtra != 0 ) {
	ArtePointer<RecoTrack> rTrack =
	  *(ArteTable<RecoTrack>::begin()
	    + ( (*rrel)->rtra - *ArteTable<RTRA>::begin() ) );
	//recoV_OutLink.setRelation( rVertex, rTrack );
	createRelation( rVertex, rTrack, &RecoVertex::out );
      }
#endif
      chi2 += (*rrel)->chi;
    }
    rVertex->chi2 = chi2;
  }

  //
  // MC vertices
  //
  for( ArteTableIterator<MVER> mver = ArteTable<MVER>::begin();
       mver != ArteTable<MVER>::end();
       ++mver ) {
    MC_Vertex mcVert( *mver );
    mcVertices->push_back( &mcVert  );
  }

  //
  // MC_Track
  //
  for( ArteTableIterator<MTRA> mtra = ArteTable<MTRA>::begin();
       mtra != ArteTable<MTRA>::end();
       ++mtra ) {

    ArtePointer<MVER> mver = (*mtra)->bver;
    ArtePointer<MC_Vertex> mVertex = 
      *(ArteTable<MC_Vertex>::begin() + (mver - *ArteTable<MVER>::begin() ) );

    MC_Track mTrack( *mtra, mVertex );
    ArtePointer<MTRA> btra = (*mtra)->btra;
    mTrack.mother = 
      *(ArteTable<MC_Track>::begin() + (btra - *ArteTable<MTRA>::begin() ) );
    
    mcTracks->push_back( &mTrack );

#ifdef USE_ROOT
    ArtePointer<MC_Track> mTrackAP = *(ArteTable<MC_Track>::end() - 1 );
    
    for( ArteRelationIterator<RTRA> rtra = (*mtra)->rtra.begin();
	 rtra != (*mtra)->rtra.end();
	 ++rtra ) {
      ArtePointer<RecoTrack> rTrack
	= *(ArteTable<RecoTrack>::begin()
	    + ( *rtra - *ArteTable<RTRA>::begin() ) );
      createRelation( rTrack, mTrackAP, &MC_Track::rTrack );
      //recoT_MC_T_Link.setRelation( mTrackAP, rTrack );
    }
#endif
  }
  return 0;
}
