
#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include "arte/Arte.h"
#include "auto/MEMO.h"
#include "auto/MEMA.h"
#include "auto/MEMS.h"
#include "auto/MEIO.h"
#include "dbarte/DbArteBridge.h"
#include "arte/ArteFlag.h"
#include "arte/mecCount.h"


extern DbFileDesc *DbFileDescList;
extern int DbLedaCompatibility;

void mmbook_(int *, int *), mmrupd_(int *), mmdrop_(int *), mmfree_(int *);
void chhtot_(char *, char *, int *);
void iounit_(char *, int *, int);
int  mmrstat_(int *, int *);
int  mmwstat_(int *, int *);
DbTableDesc *DbArteTableDesc(char *);



void DbWriteTable(int lun, int *Mlink, int major, int minor, int *ierr) 
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   * 12.08.98: Modified by A. Semenov (JINR, Dubna)
   *
   * Description: Write to a MIZZI file a single bank.
   *
   * Called from: anywhere.
   *
   * External references: MIZZI functions.
   *
   * Argument in/out description
   *  lun     in     LUN
   *  Mlink   in     bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  ierr    out    operation status
   *              0  OK
   *              1  Mlink not inside link area
   *              2  Bank name too long for MIZZI
   *              3  LUN is not connected to data file/server
   *              4  Can not book table
   *              5  Table already exists (at interactive mode, user 
    					refused overwriting)
   *              6  Can not allocate memory for buffer
   *              7  field type not implemented 
   *              8  lun is opened read-only
   *
   ************************************************************************/

  int n, m, j, nlink, *Link, Tab, FieldType, Nrows, *databuffer;
  short *databuffer1;
  char mess[80]="DBWTAB: ", *warn, what;
  char Name[DB_MAX_NAME_SIZE+1];
  char Csym[DB_MAX_NAME_SIZE+1];
  Tables Id;
  int var;
  DbTableDesc *TD;
  DbFileDesc  *FD;

  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;

  nlink = (int)Mlink / 4 - artelink_.larln1;
  
  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    static int errPtr;
    strcat( mess, "link not inside link area" );
    mecErrorCount( &errPtr, mess );
    *ierr=1;
	return;
  }

  var = DB_MAX_NAME_SIZE/4;
  chhtot_(MEMO(nlink).name, Name, &var );

  /*
  ** Check bank name length. Note that MIZZI allows only 16 characters names.
  */
  
  if( strlen(Name) > DB_MAX_TABLE_NAME_SIZE ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank name \"%s\" is too long for MIZZI (NameLen <= %d)",
       Name,DB_MAX_TABLE_NAME_SIZE);
    mecErrorCount( &errPtr, mess );
    *ierr=2;
	return;
  }
  
  /*
   * Get bank fields.
   */
  
  TD=DbArteTableDesc(Name);
   
  /*
   * Memory address of the bank. Points directly to the columns number.
   */
  
  Link  =    gcbank_.iq + *Mlink;

  /* File id (given by OpenTables). */
  
  
  for(FD=DbFileDescList;FD;FD=FD->next)
	  if(FD->lun==lun)
		  break;

  if( FD == NULL ) 
    {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "symbolic name %s is not connected to a file/server", Csym );
    mecErrorCount( &errPtr, mess );
	*ierr=3;
    return;
    }
    
  if( FD->rwflag == 0 ) 
    {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "attemt to write to read-only stream %s", Csym );
    mecErrorCount( &errPtr, mess );
	*ierr=8;
    return;
    }  

  Id   =    FD->Id;
      
  /*
  ** Book Table T->Name
  */
    
  /* Check Format string */
  
  Tab=0;
  if(SearchTable(Id, 0, TD->Name, TD->Name))
	  Tab  =  BookTable( Id, TD->Name, 0, TD->Comm );
	  
  if (Tab != 0)
     {
     char *oldFmt;
     
     oldFmt = TableDescription( Id, Tab );
     if (strlen(oldFmt) != strlen(TD->Fmt) || strcmp(oldFmt,TD->Fmt) != 0)
        {
        static int errPtr;
        char       warnMess[80];
        
        /* Warning if new description conflicts with the old one */
        if (strlen(TD->Fmt) < strlen(oldFmt) ||
            strncmp(TD->Fmt,oldFmt,strlen(oldFmt)) != 0)
           {
           sprintf(warnMess,
              "DBWTAB: bank %s: new description conflicts with the old one",
				    TD->Name );
           mecWarningCount( &errPtr, warnMess );
		   printf("Old format: '%s'\n",oldFmt);
		   printf("New format: '%s'\n",TD->Fmt);
           }
        
        UnbookTable( Id, Tab );
        Tab = 0;
        }
     }

  /* Book Table again */
    
  if (Tab == 0)
     Tab  =    BookTable( Id, TD->Name, TD->Fmt, TD->Comm );
  if( Tab == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "cannot book table %s", TD->Name );
    mecErrorCount( &errPtr, mess );
    *ierr=4;
	return;
  }
  
    
  TableStamp( Id, major , minor );
    
    
  /*
  ** Check if Major and Minor revision numbers have already been used.
  */
    
  if( MatchTable( Id, Tab, 0 ) )
    {
    if (arteflag_.ibatch == 0) /* (for interactive application only) */
      {
      sprintf(mess+strlen(mess),
         "table %s with major and minor %d %d already exists",
          Name , major, minor );
      mecAllPrint( mess );
      warn = "Overwrite y/n ?";
      mecAllPrint( warn );
      do
        {
        scanf( "%c", &what );
        what = toupper ( what );
        } while ( what != 'Y' && what != 'N' );
    
      if( what == 'N' )
        {
        UnbookTable( Id, Tab );
		*ierr=5;
        return;
        }
      }

    sprintf(mess, "DBWRIT: overwriting table %s", Name );
    if (arteflag_.ibatch == 0)
      mecAllPrint( mess );
    else
      {
      static int errPtr;
      mecWarningCount( &errPtr, mess );
      }
    }

  SkipTableField( Id, 1 );
  
  if(mmwstat_(&lun, &nlink)==0)
	  Nrows = N_U(Link);
  else
	  Nrows=0;
  
  /* Calculate data buffer size, allocate memory */
  
  var=(2+TD->Entries)*sizeof(int);
  for(n=0;n<TD->Entries;n++)
  	{
  	m=Nrows*(TD->Dim[n]==0?1:TD->Dim[n])*sizeof(int);
  	if(m>var)
  		var=m;
  	}
  	
  
  databuffer=(void *)malloc(var);
  if(databuffer==NULL)
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "can not allocate memory for buffer");
      mecErrorCount( &errPtr, mess );
      UnbookTable(Id,Tab);
	  *ierr=6;
      return;
      }
	  
   databuffer1=(short *)databuffer;

   
   if(!DbLedaCompatibility)
   {
	   
    /* Put rows number and fields dimensions into the first field. */


	  databuffer[0]=Nrows;
	  for(n=0;n<TD->Entries;n++)
  		databuffer[n+1]=TD->Dim[n];

	  PutTableField( Id, TABLEint, TD->Entries+1, databuffer );
  
   }
  
  if(Nrows==0)
  {
	StoreTable( Id, Tab, 0 );
  	UnbookTable( Id, Tab );
  	*ierr=0;
  	return;
  }
  
  /* Finally record the bank into the db file. */
  
  /* Cycle on the fields of the bank. */
  
  for( n = 0, j = 0 ; n < TD->Entries ; n++ ) {

    switch( TD->Type[n] ) {
      
    case 'I':
      FieldType = TABLEint;
      break;
      
    case 'F':
      FieldType = TABLEfloat;
      break;
      
    case 'B':
      FieldType = TABLEuint;
      break;
      
    case 'H':
      FieldType = (DbLedaCompatibility?TABLEbyte:TABLEascii);
      break;
	  
	case 'U':
	  FieldType = TABLEushort;
	  break;
	  
	case 'S':
	  FieldType = TABLEshort;
	  break;
	  
	case 'R':
	  FieldType = 0;
	  break;
      
    default:
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "wrong field type \"%c\" in table %s",
         TD->Type[n], TD->Name );
      mecErrorCount( &errPtr, mess );
	  *ierr=7;
      return;
      }

    }
    
    /* Cycle on the dimension of each field. */
	/* Procedure differs in the cases of Leda/nonLeda */
    
	if(DbLedaCompatibility==0)
	{ /* DbArte case */
	
    	for( m = 0 ; m < ( (TD->Dim[n] == 0) ? 1 : TD->Dim[n] ) ; m++ ) 
    	{
    	int k;

    	  int nCopy;

    	  nCopy=Nrows;
    	  if(TD->Type[n]=='H')
      		nCopy*=4;

		  if(TD->Type[n]=='S' || TD->Type[n]=='U')
	  		{
	  		for(k=0;k<Nrows;k++)
      			databuffer1[k*(TD->Dim[n] == 0 ? 1 : TD->Dim[n] )+m] =
      				*(Link + 2 + j + k*MEMO(nlink).col + m );
  			}
		  else
	    	{
        	for(k=0;k<Nrows;k++)
      		  databuffer[k*(TD->Dim[n] == 0 ? 1 : TD->Dim[n] )+m] =
      			*(Link + 2 + j + k*MEMO(nlink).col + m );
	    	}

    	}

    	PutTableField(Id, FieldType, 
    	  Nrows*(TD->Type[n]=='H'?4:1)*(TD->Dim[n]==0?1:TD->Dim[n]), 
      		databuffer);
		
	}
	else if (TD->Type[n]!='R')
	{ /* Leda case */
	    
	  for( m = 0 ; m < ( (TD->Dim[n] == 0) ? 1 : TD->Dim[n] ) ; m++ ) 
	  {
 
      int nCopy;
 
      nCopy = N_U(Link);
 
      if(TD->Type[n]!='H')
	  {
		  if(TD->Type[n]=='S' || TD->Type[n]=='U')
		  {
			  int k;
			 for(k=0;k<Nrows;k++)
      				databuffer1[k*(TD->Dim[n] == 0 ? 1 : TD->Dim[n] )+m] =
      					*(Link + 2 + j + k*MEMO(nlink).col + m );
			  PutTableField(Id, FieldType, 
    		  Nrows*(TD->Type[n]=='H'?4:1)*(TD->Dim[n]==0?1:TD->Dim[n]), 
      			databuffer);
		  }
		  else
    	  PutTableStruct( Id, FieldType, MEMO(nlink).col * 4, nCopy,
                    	  (void *)(Link + 2 + j + m));
      }
      else
      {
              PutTableStruct( Id, FieldType, MEMO(nlink).col * 4, nCopy,
                  ((void *)(Link + 2 + j + m) + 0));
              PutTableStruct( Id, FieldType, MEMO(nlink).col * 4, nCopy,
                  ((void *)(Link + 2 + j + m) + 1));
              PutTableStruct( Id, FieldType, MEMO(nlink).col * 4, nCopy,
                  ((void *)(Link + 2 + j + m) + 2));
              PutTableStruct( Id, FieldType, MEMO(nlink).col * 4, nCopy,
                  ((void *)(Link + 2 + j + m) + 3));
      }
 
      }
		
	}
    
    j += ( (TD->Dim[n] ==0 ) ? 1 : TD->Dim[n] );
    
  }
  StoreTable( Id, Tab, 0 );
  UnbookTable( Id, Tab );
  *ierr=0;
  return;
  
}

