#include "arte/arte.hh"
#include "rich/ricutil.h"
#include "rich/vessel.h"
#include "CLHEP/Vector/ThreeVector.h"

extern "C" {

  void kugetr_(float *);
  float ranmar_(float *vec, int &len);
  void atstopc_();
  
  void phofrac_() {  
    kugetr_(&RichRun::photon_fraction);
    if ( RichRun::photon_fraction>1 || RichRun::photon_fraction<1e-3) {
      cerr<<" === gasfix: Photon fraction in RICH out of range,"<<endl;
      cerr<<" === gasfix: please use HELP command for correct usage!"<<endl;
      cerr<<" === gasfix: Exiting ... "<<endl;
      atstopc_();
    }
    cout<<" === RICH photon yield set to "<<RichRun::photon_fraction*100<<"% of full freon tank"<<endl; 
    cout<<" === RICH ring radius set to "<<sqrt(RichRun::photon_fraction)*100<<"% of full freon tank"<<endl; 
  }
  
  
  void n2frac_() {  
    float N2_fraction;
    kugetr_(&N2_fraction);
    if (N2_fraction>1) {
      cerr<<" === N2 fraction in RICH too large"<<endl;
      cerr<<" === please use HELP command for correct usage!"<<endl;
      cerr<<" === Exiting ... "<<endl;
      atstopc_();
    }
    RichRun::photon_fraction=1-N2_fraction + N2_fraction*.205; // (theta_N2/theta_fraon)^2=0.205
    cout<<" === RICH N2 concentration set to "<<N2_fraction*100<<"%"<<endl; 
    cout<<" === RICH photon yield set to "<<RichRun::photon_fraction*100<<"% of full freon tank"<<endl; 
    cout<<" === RICH ring radius set to "<<sqrt(RichRun::photon_fraction)*100<<"% of full freon tank"<<endl; 
  }

	   }

	   


int gasfix(Photon &ph, ArtePointer<MIMP> mimp) {

 
  ///////////// cut number of photons ////////////////
  if (RichRun::photon_fraction>.999) return 0; // no cut
  int len=1;
  float r;
  ranmar_(&r,len);
  if (r>RichRun::photon_fraction) return 1; // cut

  /////////////  reduce radius size  /////////////////
  static int err_count=0;
  if (fabs(mimp->q)<1e-8) {
    if (err_count<10) {
      cerr<<" === gasfix: Polarization vector of RICH photon not set properly!"<<endl;
      cerr<<" === gasfix: Skipping radius resizing (photon number cut only)!"<<endl;
      if (!err_count) cerr<<" === gasfix: the message will be displayed 10 times only"<<endl;
      err_count++;
    }
    return 0; // OK
  }

  // old Ch. angle
  double lambda           = energy2lambda(ph.e());
  double refraction_index = RICH_Vessel->Radiator().RefractiveIndex(lambda);
  ArtePointer<MTRA> mtra  = mimp->mtra;
  double P=sqrt(mtra->px*mtra->px+mtra->py*mtra->py+mtra->pz*mtra->pz);
  double mass=mimp->mtra->topp->mass;
  double beta=P/sqrt(P*P+mass*mass);
  double theta=acos(1/(beta*refraction_index));
  
  // new Ch. angle
  double new_theta      = sqrt(RichRun::photon_fraction)*theta;
  double rotation_angle = new_theta-theta;
  Vec3   rotation_axis  = normalize(cross(ph.n(), ph.polarisation()));

  // new photon direction/polarization
  Hep3Vector n(ph.n().x(),ph.n().y(),ph.n().z());
  Hep3Vector p(ph.polarisation().x(),ph.polarisation().y(),ph.polarisation().z());
  Hep3Vector t(rotation_axis.x(), rotation_axis.y(), rotation_axis.z());
  Hep3Vector new_n = n;
  new_n.rotate(rotation_angle, t);
  Hep3Vector new_p = p;
  new_p.rotate(rotation_angle, t);

  // change Photon
  ph.n()           =Vec3(new_n.x(), new_n.y(), new_n.z());
  ph.polarisation()=Vec3(new_p.x(), new_p.y(), new_p.z());

  // change MIMP
  mimp->tx   = new_n.x()*ph.e();
  mimp->ty   = new_n.y()*ph.e();
  mimp->p    = new_n.z()*ph.e();
  mimp->time = new_p.x();
  mimp->q    = new_p.y();

  return 0;
}


