/************************************************************************
*
* dgmupix.h
* 
* first author : Jing-Ge Shiu 
* purpose      : C++/SLT code for digitizing MC MUON pixel hits.
*                This routines translates GEANT output hits into 
*                ARTE HITB for inner muon pixel chamber
* called from  : dgmuoic.C
* start date   : Jan. 25, 2000
*
*
*-----------------------------------------------------------------------
*
* ver 0.00     : finished at Feb. 25, 2000
* ver 1.00     : fix position caculation (remove 0.5*pix_[xy]siz term)
*                count the speical virtual pad structure
*                finished at Mar. 2, 2000
*
*
*
*/


#include <iomanip.h>
#include "arte/arte.hh"
#include "arte/ArteTable.hh"
#include "auto/auto.hh"

#include "murec/MuDbPixelArte.h"

static MuPixChamber PixCh;
const int MaxPixCol = 48;
const int MaxPixRow = 36;
const int MaxVpCol = 48;
const int MaxVpRow = 17;
const int MaxNhit = 20;      

#define MAX(a,b)  a>b?a:b
#define MIN(a,b)  a<b?a:b

struct TWOINT {
  int i1, i2;  
};

struct TWOFLOAT {
  float f1, f2;
};


class MuPixGEDEVp {

  public :
     MuPixGEDEVp(); 
     ~MuPixGEDEVp();
     void printpixvp(int, int); 

  private :

};


class MuPixGEDE {
  public:
     MuPixGEDE(ArtePointer<GEDE>);              // constructor
     MuPixGEDE(const MuPixGEDE&);               // copy constructor
     ~MuPixGEDE();                              // destructor
     bool ingede(ArtePointer<MIMP>,int*);       // is this mimp hit in the gede area?
     void fillhitb(void);                       // fill HITB 
     void printptn(void);                       // print out the hit pattern

  protected:
  private:

      void cleargedehit(void);
      bool hitvp(int, int);
      bool gedeeff(int, int);
      bool squarehit(TWOFLOAT&, TWOFLOAT&);
      bool special3(int, int);
      bool special3hit(int, int, bool&, TWOFLOAT&, TWOFLOAT&);
      TWOFLOAT vp_xsides(int,int);
      TWOFLOAT vp_ysides(int,int);
      TWOINT xy_vp(float, float);
      
      ArtePointer<GEDE> gede;
      ArtePointer<MIMP> mimp;

      TWOINT pix_vp[MaxPixCol][MaxPixRow];
      int sl, subl;
      int Npvx,Npvy,STEPx,STEPy;
      float pix_xref, pix_yref, pix_zref, pix_xsiz, pix_ysiz;
      float vp_xsiz,vp_ysiz;
      float zmin, zmax;
      float mimp_x, mimp_y, mimp_z, mimp_tx, mimp_ty, mimp_tyx, mimp_txy;
      float mimp_xin, mimp_xout,mimp_yin,mimp_yout;
      int   gedehit[48][17];
      ArtePointer<MIMP> hitmimp[MaxVpCol][MaxVpRow][MaxNhit];     

};



#include "dgmupix.icc"











