//                              -*- Mode: C++ -*-
//
// Particle.h - base class particle
//
// Copyright (C)  1993  The Board of Trustees of The Leland Stanford
//                      Junior University.  All Rights Reserved.
//
// $Id: DQ_Particle.hh,v 1.1 2000/04/19 20:42:16 natasha Exp $

#ifndef _PARTICLE_H_
#define _PARTICLE_H_

#ifdef __GNUC__
#pragma interface
#endif

#include <CLHEP/Vector/LorentzVector.h>
#include <CLHEP/Alist/AList.h>
#include "iostream.h"

typedef void* ClassId;

class Particle : public HepLorentzVector
{
public:
     Particle() {_pdtEntry = 0; _charge = 0; _parent = 0; }
     Particle(const HepLorentzVector &);
//     Particle(HepLorentzVector &, PDTEntry *);
     Particle(const Particle &);
     virtual ~Particle() {}

     virtual int isA(ClassId id) { return id == ClassID(); }
     static ClassId ClassID() { static int x; return &x; }

     float charge() const { return _charge; }
//     long idCode() const { return _pdtEntry ?_pdtEntry->idCode() : 0; }
//     PDTEntry *particleData() const { return _pdtEntry; }
     float mass() const { return mag(); }

     // parent and children
     int nChildren() const { return _children.length(); }
     Particle *child(int i) const { return _children[i]; }
     Particle *parent() const { return _parent; }
     void setParent(Particle *p) { _parent = p; }
     void addChild(Particle *p) { _children.append(p); }
     void clearChildren() { _children.removeAll(); }

     // particle ID methods
//     virtual void setIDAdjustE(PDTEntry *);
//     virtual void setIDAdjustP(PDTEntry *);

     // Addition operators.
     virtual Particle& operator += (const Particle &);
     virtual Particle operator+ (const Particle &);

     // assignment
     virtual Particle & operator = (const Particle &);

     virtual void printOn(ostream &str) {
          str << "Charge " << charge() << ": " << *(HepLorentzVector*)this;
     }
     void printAncestry(ostream &str, int i=0);



protected:
  float _charge;             // units of e
//     PDTEntry *_pdtEntry;
  int* _pdtEntry;
  HepAList<Particle> _children;
  Particle *_parent;

};

inline ostream& operator<<(ostream& str, Particle& p)
{
     p.Particle::printOn(str);          // don't use virtual function!!
     return str;
}

inline Particle::Particle(const Particle &p)
: HepLorentzVector(p),  _children(p._children){
     _charge = p._charge;
     _pdtEntry = p._pdtEntry;
     _parent = p._parent;
}

inline Particle& Particle::operator += (const Particle &p)
{
     HepLorentzVector::operator+=(p);
     _charge += p._charge;
     _pdtEntry = NULL;
     return *this;
}

inline Particle Particle::operator + (const Particle &p)
{
  HepLorentzVector vect(*(HepLorentzVector *)this+(HepLorentzVector &)p);
  Particle p2(vect);
  p2._charge = _charge + p._charge;
  return p2;
}

inline Particle& Particle::operator = (const Particle &p)
{
     HepLorentzVector::operator=(p);
     _charge = p._charge;
     _pdtEntry = p._pdtEntry;
     _children = p._children;
     _parent = p._parent;
     return *this;
}

#endif                          // _PARTICLE_H_




