#include <iostream>
#include <iomanip>
#include <string>
#include "grover/DumpFormatter.hh" //  <== Information here

//
// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
//
//

// Define some formatting stuff
#define TYPE_FIELDWIDTH   15
#define NAME_FIELDWIDTH   15
#define VALUE_FIELDWIDTH  38
#define FULL_FIELDWIDTH   72
#define TEXT_FIELDWIDTH   68
#define FLOAT_FIELDWIDTH   9
#define FLOAT_FIELDWIDTH2  13
#define LEFT_MARKER       "* "
#define RIGHT_MARKER      " *"
#define TOP_MARKER        \
 "************************************************************************"
#define BOTTOM_MARKER     \
 "************************************************************************"

// Default Constructor and Destructor
DumpFormatter::DumpFormatter(){};
DumpFormatter::~DumpFormatter(){};

// Member functions start here

void DumpFormatter::DumpClassName(char classname[])
{
  cout << TOP_MARKER   << endl;
  cout << LEFT_MARKER << 
    setw(TEXT_FIELDWIDTH) << "" << 
    RIGHT_MARKER << endl;
  cout << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH)  << "PARTIAL DUMP OF" <<
    setw(NAME_FIELDWIDTH)  << " CLASS =======>" <<
    setw(VALUE_FIELDWIDTH) << classname   << RIGHT_MARKER << endl;
  cout << LEFT_MARKER << 
    setw(TEXT_FIELDWIDTH) << "" << 
    RIGHT_MARKER << endl;
}


void DumpFormatter::DumpComment(char comment[])
{
  cout << LEFT_MARKER << 
    setw(TEXT_FIELDWIDTH) << comment << 
    RIGHT_MARKER << endl;
}


void DumpFormatter::DumpHeader()
{
  cout << LEFT_MARKER << setw(TEXT_FIELDWIDTH) << ""  << RIGHT_MARKER << endl;
  cout << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH)   << "Type"  <<
    setw(NAME_FIELDWIDTH)   << "Name"  <<
    setw(VALUE_FIELDWIDTH) << "Value" << RIGHT_MARKER << endl;
  cout << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH)   << "----"  <<
    setw(NAME_FIELDWIDTH)   << "----"  <<
    setw(VALUE_FIELDWIDTH) << "-----" << RIGHT_MARKER << endl;
}


void DumpFormatter::DumpFooter()
{
  cout << LEFT_MARKER << 
    setw(TEXT_FIELDWIDTH) << "" << 
    RIGHT_MARKER << endl;
  cout << BOTTOM_MARKER << endl;
}


void DumpFormatter::DumpPrint(char varname[], ...)
{ // DEFAULT (safe)
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "?"     <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH)  << "?"     << 
    RIGHT_MARKER << endl;
}

// Integer and derivatives

void DumpFormatter::DumpPrint(char varname[], int *var)
{ // Integer
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "int"   <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH) << *var    << 
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], unsigned int *var)
{ // Unsigned Integer
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "unsigned int"   <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH) << *var    << 
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], short *var)
{ // Short
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "short"   <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH) << *var    << 
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], unsigned short *var)
{ // Unsigned Short
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "unsign. short"   <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH) << *var    << 
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], long *var)
{ // Long
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "long"   <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH) << *var    << 
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], unsigned long *var)
{ // Unsigned Long
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "unsigned long"   <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH) << *var    << 
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], long long *var)
{ // Long Long
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "long long"   <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH) << *var    << 
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], unsigned long long *var)
{ // Unsigned Long Long
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "uns. long long"   <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH) << *var    << 
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], unsigned long long var[], int size)
{ // Unsigned Long Long array. not so ugly, but complicated
  //with explicit cast to float
  cout.setf(ios::scientific, ios::floatfield); //scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  //  int width = FLOAT_FIELDWIDTH; //width of numbers eg "1.1E-06" = 7 chars
  int i=0;
  int k=0;
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "uns. l. long[]" <<
    setw(NAME_FIELDWIDTH)   << varname << "  ";    
  while (i<4){
    if ((i == 2) && (size > 4)){
      cout <<
	setw(FLOAT_FIELDWIDTH) << "<...> " << 
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+size-1));
      break;
    };
    if (3-size-i >= 0) cout << setw(FLOAT_FIELDWIDTH) << "";
    else{
      cout <<
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+k));
      k=k+1;
    }
  i = i+1;
  };
  cout << RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], long long var[], int size)
{ // Long Long array. not so ugly, but complicated
  //with explicit cast to float
  cout.setf(ios::scientific, ios::floatfield); //scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  //  int width = FLOAT_FIELDWIDTH; //width of numbers eg "1.1E-06" = 7 chars
  int i=0;
  int k=0;
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "long long[]" <<
    setw(NAME_FIELDWIDTH)   << varname << "  ";    
  while (i<4){
    if ((i == 2) && (size > 4)){
      cout <<
	setw(FLOAT_FIELDWIDTH) << "<...> " << 
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+size-1));
      break;
    };
    if (3-size-i >= 0) cout << setw(FLOAT_FIELDWIDTH) << "";
    else{
      cout <<
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+k));
      k=k+1;
    }
  i = i+1;
  };
  cout << RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], short var[], int size)
{ // Short array. not so ugly, but complicated
  //with explicit cast to float
  cout.setf(ios::scientific, ios::floatfield); //scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  //  int width = FLOAT_FIELDWIDTH; //width of numbers eg "1.1E-06" = 7 chars
  int i=0;
  int k=0;
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "short[]" <<
    setw(NAME_FIELDWIDTH)   << varname << "  ";    
  while (i<4){
    if ((i == 2) && (size > 4)){
      cout <<
	setw(FLOAT_FIELDWIDTH) << "<...> " << 
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+size-1));
      break;
    };
    if (3-size-i >= 0) cout << setw(FLOAT_FIELDWIDTH) << "";
    else{
      cout <<
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+k));
      k=k+1;
    }
  i = i+1;
  };
  cout << RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], unsigned short var[], int size)
{ // Unsigned Short array. not so ugly, but complicated
  //with explicit cast to float
  cout.setf(ios::scientific, ios::floatfield); //scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  //  int width = FLOAT_FIELDWIDTH; //width of numbers eg "1.1E-06" = 7 chars
  int i=0;
  int k=0;
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "uns. short[]" <<
    setw(NAME_FIELDWIDTH)   << varname << "  ";    
  while (i<4){
    if ((i == 2) && (size > 4)){
      cout <<
	setw(FLOAT_FIELDWIDTH) << "<...> " << 
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+size-1));
      break;
    };
    if (3-size-i >= 0) cout << setw(FLOAT_FIELDWIDTH) << "";
    else{
      cout <<
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+k));
      k=k+1;
    }
  i = i+1;
  };
  cout << RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], unsigned long var[], int size)
{ // Unsigned Long array. not so ugly, but complicated
  //with explicit cast to float
  cout.setf(ios::scientific, ios::floatfield); //scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  //  int width = FLOAT_FIELDWIDTH; //width of numbers eg "1.1E-06" = 7 chars
  int i=0;
  int k=0;
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "unsigned long[]" <<
    setw(NAME_FIELDWIDTH)   << varname << "  ";    
  while (i<4){
    if ((i == 2) && (size > 4)){
      cout <<
	setw(FLOAT_FIELDWIDTH) << "<...> " << 
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+size-1));
      break;
    };
    if (3-size-i >= 0) cout << setw(FLOAT_FIELDWIDTH) << "";
    else{
      cout <<
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+k));
      k=k+1;
    }
  i = i+1;
  };
  cout << RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], long var[], int size)
{ // Long array. not so ugly, but complicated, with explicit cast to float
  cout.setf(ios::scientific, ios::floatfield); //scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  //  int width = FLOAT_FIELDWIDTH; //width of numbers eg "1.1E-06" = 7 chars
  int i=0;
  int k=0;
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "long[]" <<
    setw(NAME_FIELDWIDTH)   << varname << "  ";    
  while (i<4){
    if ((i == 2) && (size > 4)){
      cout <<
	setw(FLOAT_FIELDWIDTH) << "<...> " << 
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+size-1));
      break;
    };
    if (3-size-i >= 0) cout << setw(FLOAT_FIELDWIDTH) << "";
    else{
      cout <<
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+k));
      k=k+1;
    }
  i = i+1;
  };
  cout << RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], int var[], int size)
{ // Int array. not so ugly, but complicated, with explicit cast to float
  cout.setf(ios::scientific, ios::floatfield); //scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  //  int width = FLOAT_FIELDWIDTH; //width of numbers eg "1.1E-06" = 7 chars
  int i=0;
  int k=0;
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "int[]" <<
    setw(NAME_FIELDWIDTH)   << varname << "  ";    
  while (i<4){
    if ((i == 2) && (size > 4)){
      cout <<
	setw(FLOAT_FIELDWIDTH) << "<...> " << 
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+size-1));
      break;
    };
    if (3-size-i >= 0) cout << setw(FLOAT_FIELDWIDTH) << "";
    else{
      cout <<
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+k));
      k=k+1;
    }
  i = i+1;
  };
  cout << RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], unsigned int var[], int size)
{ // unsigned Int array. not so ugly, but complicated 
  // with explicit cast to float
  cout.setf(ios::scientific, ios::floatfield); //scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  //  int width = FLOAT_FIELDWIDTH; //width of numbers eg "1.1E-06" = 7 chars
  int i=0;
  int k=0;
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "unsigned int[]" <<
    setw(NAME_FIELDWIDTH)   << varname << "  ";    
  while (i<4){
    if ((i == 2) && (size > 4)){
      cout <<
	setw(FLOAT_FIELDWIDTH) << "<...> " << 
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+size-1));
      break;
    };
    if (3-size-i >= 0) cout << setw(FLOAT_FIELDWIDTH) << "";
    else{
      cout <<
	setw(FLOAT_FIELDWIDTH) << (float)(*(var+k));
      k=k+1;
    }
  i = i+1;
  };
  cout << RIGHT_MARKER << endl;
}

//Double and derivatives

void DumpFormatter::DumpPrint(char varname[], double *var)
{ // Double
  cout << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH)   << "double"   <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH) << *var    << 
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], long double *var)
{ // Long Double
  cout << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH)   << "long double"   <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH) << *var    << 
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], double var[], int size)
{ // Double array. not so ugly, but complicated
  cout.setf(ios::scientific, ios::floatfield); // Scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  //  int width = FLOAT_FIELDWIDTH; //width of numbers eg "1.1E-06" = 7 chars
  int i=0;
  int k=0;
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "double[]" <<
    setw(NAME_FIELDWIDTH)   << varname << "  ";    
  while (i<4){
    if ((i == 2) && (size > 4)){
      cout <<
	setw(FLOAT_FIELDWIDTH) << "<...> " << 
	setw(FLOAT_FIELDWIDTH) << *(var+size-1);
      break;
    };
    if (3-size-i >= 0) cout << setw(FLOAT_FIELDWIDTH) << "";
    else{
      cout <<
	setw(FLOAT_FIELDWIDTH) << *(var+k);
      k=k+1;
    }
  i = i+1;
  };
  cout << RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], long double var[], int size)
{ // long double array. not so ugly, but complicated
  cout.setf(ios::scientific, ios::floatfield); // Scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  //  int width = FLOAT_FIELDWIDTH; //width of numbers eg "1.1E-06" = 7 chars
  int i=0;
  int k=0;
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "long double[]" <<
    setw(NAME_FIELDWIDTH)   << varname << "  ";    
  while (i<4){
    if ((i == 2) && (size > 4)){
      cout <<
	setw(FLOAT_FIELDWIDTH) << "<...> " << 
	setw(FLOAT_FIELDWIDTH) << *(var+size-1);
      break;
    };
    if (3-size-i >= 0) cout << setw(FLOAT_FIELDWIDTH) << "";
    else{
      cout <<
	setw(FLOAT_FIELDWIDTH) << *(var+k);
      k=k+1;
    }
  i = i+1;
  };
  cout << RIGHT_MARKER << endl;
}

//Chars

void DumpFormatter::DumpPrint(char varname[], char *var)
{ // Char
  cout << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH)  << "char" <<
    setw(NAME_FIELDWIDTH)  << varname <<
    setw(VALUE_FIELDWIDTH-1) << "" << *var <<
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], unsigned char *var)
{ // Unsigned Char
  cout << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH)  << "unsigned char" <<
    setw(NAME_FIELDWIDTH)  << varname <<
    setw(VALUE_FIELDWIDTH-1) << "" << *var <<
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], char var[], int size)
{ //Char-array

  cout << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH)  << "char[]" <<
    setw(NAME_FIELDWIDTH)  << varname;
  ;
  if (size <= 37) 
    cout <<   setw(VALUE_FIELDWIDTH) << var <<
      RIGHT_MARKER << endl;
  else{
    cout << "  ";
    for(int i=0; i<32; i=i+1) cout << *(var+i);
    cout << " ..."  << RIGHT_MARKER << endl;
  };
}

void DumpFormatter::DumpPrint(char varname[], unsigned char var[], int size)
{ //unsigned Char-array

  cout << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH)  << "unsigned char[]" <<
    setw(NAME_FIELDWIDTH)  << varname;
  ;
  if (size <= 37) 
    cout <<   setw(VALUE_FIELDWIDTH) << var <<
      RIGHT_MARKER << endl;
  else{
    cout << "  ";
    for(int i=0; i<32; i=i+1) cout << *(var+i);
    cout << " ..."  << RIGHT_MARKER << endl;
  };  
}

//Float stuff

void DumpFormatter::DumpPrint(char varname[], float *var)
{ // Float
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "float" <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH) << *var    << 
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], float var[], int size)
{ // float array. not so ugly, but complicated
  cout.setf(ios::scientific, ios::floatfield); // Scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  //  int width = FLOAT_FIELDWIDTH; //width of numbers eg "1.1E-06" = 7 chars
  int i=0;
  int k=0;
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "float[]" <<
    setw(NAME_FIELDWIDTH)   << varname << "  ";    
  while (i<4){
    if ((i == 2) && (size > 4)){
      cout <<
	setw(FLOAT_FIELDWIDTH) << "<...> " << 
	setw(FLOAT_FIELDWIDTH) << *(var+size-1);
      break;
    };
    if (3-size-i >= 0) cout << setw(FLOAT_FIELDWIDTH) << "";
    else{
      cout <<
	setw(FLOAT_FIELDWIDTH) << *(var+k);
      k=k+1;
    }
  i = i+1;
  };
  cout << RIGHT_MARKER << endl;
}

//true or false?

void DumpFormatter::DumpPrint(char varname[], bool *var)
{ // Bool
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "bool"   <<
    setw(NAME_FIELDWIDTH)   << varname <<
    setw(VALUE_FIELDWIDTH) << *var    << 
    RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], bool var[], int size)
{ // Bool array. not so ugly, but complicated, with explicit cast to float
  cout.setf(ios::scientific, ios::floatfield); //scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  //  int width = FLOAT_FIELDWIDTH; //width of numbers eg "1.1E-06" = 7 chars
  int i=0;
  int k=0;
  cout << LEFT_MARKER <<  
    setw(TYPE_FIELDWIDTH)   << "bool[]" <<
    setw(NAME_FIELDWIDTH)   << varname << "  ";
  if (size > 16){
    for (int i = 0; i < 15; i = i+1) cout << *(var+i) << " ";
    cout << " ... " << *(var+size-1);
  }
  else{
    for (int i = 0; i < (36-2*size); i = i+1) cout << " ";
    for (int i = 0; i < size; i = i+1) cout << " " << *(var+i);
  };
  cout << RIGHT_MARKER << endl;
}

//more complex data-types

/*
void DumpFormatter::DumpPrint(char varname[], complex<double> *var)
{ // Complex numbers,  double
  cout << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH) << "complex no." <<
    setw(NAME_FIELDWIDTH) << varname <<
    "                     " << *var <<
    RIGHT_MARKER << endl;

}

void DumpFormatter::DumpPrint(char varname[], complex<long double> *var)
{ // Complex numbers,  long double
  cout << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH) << "complex no." <<
    setw(NAME_FIELDWIDTH) << varname <<
    "                     " << *var <<
    RIGHT_MARKER << endl;

}

void DumpFormatter::DumpPrint(char varname[], complex<float> *var)
{ // Complex numbers,  float
  cout << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH) << "complex no." <<
    setw(NAME_FIELDWIDTH) << varname <<
    "                     " << *var <<
    RIGHT_MARKER << endl;

}

void DumpFormatter::DumpPrint(char varname[], Hep3Vector *vec)
{ // CLHEP three-vector "Hep3Vector"
  cout.setf(ios::scientific, ios::floatfield); // Scientific notation
  cout.precision(3);                           // 1.123456E-02 -> 1.123E-02
  cout << LEFT_MARKER <<  
  setw(TYPE_FIELDWIDTH)   << "Hep3Vector" <<
  setw(NAME_FIELDWIDTH)   << varname  <<
  setw(FLOAT_FIELDWIDTH2+1)  << vec->x() << 
  setw(FLOAT_FIELDWIDTH2-1)  << vec->y() <<
  setw(FLOAT_FIELDWIDTH2-1)  << vec->z() <<
  RIGHT_MARKER << endl;
}

void DumpFormatter::DumpPrint(char varname[], HepLorentzVector *vec)
{ // CLHEP four-vector "HepLorentzVector"
  cout.setf(ios::scientific, ios::floatfield); // Scientific notation
  cout.precision(1);                           // 1.123456E-02 -> 1.1E-02
  cout  << LEFT_MARKER <<
    setw(TYPE_FIELDWIDTH)  <<  "HepLor.Vector" <<
    setw(NAME_FIELDWIDTH)  <<  varname << "  " <<
    setw(FLOAT_FIELDWIDTH) << vec->x() <<
    setw(FLOAT_FIELDWIDTH)   << vec->y() <<
    setw(FLOAT_FIELDWIDTH)   << vec->z() <<
    setw(FLOAT_FIELDWIDTH) << vec->t() <<
    RIGHT_MARKER << endl;
}


void DumpFormatter::DumpPrint(char varname[], HepVector3D *vecty)
{ // CLHEP three-vector "HepVector3D"
  cout.setf(ios::scientific, ios::floatfield); // Scientific notation
  cout.precision(3);                           // 1.123456E-02 -> 1.123E-02
 cout << LEFT_MARKER <<  
  setw(TYPE_FIELDWIDTH)   << "HepVector3D" <<
  setw(NAME_FIELDWIDTH)   << varname  <<
  setw(FLOAT_FIELDWIDTH2+1)  << vecty->x() << 
  setw(FLOAT_FIELDWIDTH2-1)  << vecty->y() <<
  setw(FLOAT_FIELDWIDTH2-1)  << vecty->z() <<
  RIGHT_MARKER << endl;
}

*/
