#include "arte/arte.hh"
#include "grover/grover.hh"
#include "grover/higz.h"

//
// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// YY-MM-DD  Name     Modification
// -----------------------------------------------------------------------------
//
//


////////////////////////////////////////////////////////////////////////////


int graph_ini(void)
{
  // graph_ini: Initialise HIGZ for GroverPrimaries display and
  // draw the beam spot.
  // Called automatically when GroverPrimaries called for the
  // first time.
  iginit(0);
  igsse(0, 7); // Setup (7= workstation type, 0=LUN for error messages)
  iscr(1, Black       ,0.0000, 0.0000, 0.0000); // define the colours
  iscr(1, DarkGray    ,0.5000, 0.5400, 0.5300); // "Black" etc defined in 
  iscr(1, Blue        ,0.0000, 0.0000, 1.0000); // Grover header file
  iscr(1, Green       ,0.1333, 0.5451, 0.1333);
  iscr(1, Cyan        ,0.0000, 0.8078, 0.8196);
  iscr(1, Red         ,1.0000, 0.0000, 0.0000);
  iscr(1, Magenta     ,1.0000, 0.0000, 1.0000);
  iscr(1, Brown       ,0.5000, 0.1647, 0.1647);
  iscr(1, LightBlue   ,0.6784, 0.8471, 0.9020);
  iscr(1, LightGreen  ,0.5961, 0.9843, 0.5961);
  iscr(1, LightCyan   ,0.4980, 1.0000, 0.8314);
  iscr(1, LightRed    ,0.9804, 0.5020, 0.4471);
  iscr(1, LightMagenta,0.8549, 0.4392, 0.8392);
  iscr(1, Yellow      ,1.0000, 1.0000, 0.0000);
  iscr(1, LightGray   ,0.8275, 0.8275, 0.8275);
  iscr(1, White       ,1.0000, 1.0000, 1.0000);

  float beam_x = 0.30, beam_y = -0.05;  // beam center for simulated data
  if (!ArteTable<MCIN>::booked()){
    beam_x =  0.30;                     // beam center for real data
    beam_y = -0.05;
  }

  float size = 0.8;

  // Define coordinate system of window
  iswn(2, beam_x-(size/2.0), beam_x+(size/2.0), 
          beam_y-(size/2.0), beam_y+(size/2.0) );
  isvp(2, 0.0, 1.0, 0.0, 1.0);
  iselnt(2);
  // Set some attributes
  isfais(1); // no hatching
  isfaci(White); // Colour of fill area
  isplci(Red); // colour of polylines
  igbox(beam_x-(size/2.0), beam_x+(size/2.0), 
	beam_y-(size/2.0), beam_y+(size/2.0));// draw box

  iuwk(1,1); // Update canvas
  return 0;
}


/////////////////////////////////////////////////////////////////////////


void GroverKillGraphics(void)
{
  // Should be called in usstop() although it seems
  // harmless just to quit arte...
  cout << "*** Grover: shutting down HIGZ display" << endl;
  igend();
}


/////////////////////////////////////////////////////////////////////////


void draw_targets(GroverWireList* wires)
{  
  float xpl[5], ypl[5];
  isfaci(Black);
  isplci(Red);
       
  iclrwk(1,0); // clear canvas

  for (GroverWireList::iterator tarit = wires->begin();
      tarit != wires->end(); ++tarit) {
    float xt   = tarit->X();
    float yt   = tarit->Y();
    float zt   = tarit->Z();
    float dxt  = tarit->SizeX();
    float dyt  = tarit->SizeY();
    //dxt = 0.1; dyt = 0.1; // hack
    // wires are yellow and 0.4cm long
    isplci(Black);
    if (zt>-2.5) isplci(Black);
    if (tarit->Horizontal())
      {
	xpl[0] = xt-0.2;
	ypl[0] = yt;
	xpl[1] = xt+0.2;
	ypl[1] = yt;
      }
    else
      {
	xpl[0] = xt;
	ypl[0] = yt-0.2;
	xpl[1] = xt;
	ypl[1] = yt+0.2;
      }
    ipl(2,xpl,ypl);
    // station I or II ?
    if (zt>-2.5)
      isplci(Yellow);
    else
      isplci(Green);
    if (false) {
      xpl[0] = xt-dxt;
      ypl[0] = yt-dyt;
      xpl[1] = xt+dxt;
      ypl[1] = yt-dyt;
      xpl[2] = xt+dxt;
      ypl[2] = yt+dyt;
      xpl[3] = xt-dxt;
      ypl[3] = yt+dyt;
      xpl[4] = xt-dxt;
      ypl[4] = yt-dyt;
      ipl(5,xpl,ypl);
    }
  }
  iuwk(1,1); // update canvas

}


////////////////////////////////////////////////////////////////////////////


void draw_mc(float zplane)
{

  float xpm[1], ypm[1];

  // Draw MC tracks with >=5 hits in VDS
  ismk(3);
  ismksc(0.5);
  ispmci(Cyan);
  for (ArteTable<MTRA>::iterator mtraIt = ArteTable<MTRA>::begin();
       mtraIt != ArteTable<MTRA>::end(); ++mtraIt){
    if ((*mtraIt)->topp->chrg == 0 )
      continue;
    int nhitb = 0;
    // Draw 
    for ( ArteRelationIterator<MIMP> mimpIt = (*mtraIt)->mimp.begin();
	  mimpIt != (*mtraIt)->mimp.end(); ++mimpIt){
      if ((*mimpIt)->gede->cmp != 1) continue;
      nhitb += distance((*mimpIt)->hitb.begin(),(*mimpIt)->hitb.end()); 
    }
    if (nhitb<95)
      continue;
    float px = (*mtraIt)->px / (*mtraIt)->pz;
    float py = (*mtraIt)->py / (*mtraIt)->pz;
    float xm = (*mtraIt)->bver->x;
    float ym = (*mtraIt)->bver->y;
    float zm = (*mtraIt)->bver->z;
    float zt = zplane;
    xpm[0]  = xm+(zt-zm)*px;
    ypm[0]  = ym+(zt-zm)*py;
    ipm(1,xpm,ypm);
  }

  // Draw MC vertices with >=3 tracks with >=7 hits
  ismk(28);
  ismksc(5.0);
  for( ArteTable<MVER>::iterator mverIt = ArteTable<MVER>::begin();
       mverIt != ArteTable<MVER>::end();++mverIt){
    if( (*mverIt)->flag != 1) continue;
    int mtrafound = 0;
    for (ArteTable<MTRA>::iterator mtraIt = ArteTable<MTRA>::begin();
	 mtraIt != ArteTable<MTRA>::end(); ++mtraIt){
      if ((*mtraIt)->bver != (*mverIt)) continue;
      int nhitb = 0;
      for ( ArteRelationIterator<MIMP> mimpIt = (*mtraIt)->mimp.begin();
	    mimpIt != (*mtraIt)->mimp.end(); ++mimpIt){
	if ((*mimpIt)->gede->cmp != 1) continue;
	nhitb += distance((*mimpIt)->hitb.begin(),(*mimpIt)->hitb.end()); 
      }
      if (nhitb>=7){
	mtrafound++;
      }
    }
    if (mtrafound<3) continue;
    if ((*mverIt)->z>-2.5)
      ispmci(Green);
    else
      ispmci(Green);
    xpm[0]=(*mverIt)->x;
    ypm[0]=(*mverIt)->y;
    ipm(1,xpm,ypm);
  }

  iuwk(1,1); // update canvas
}


//////////////////////////////////////////////////////////////////////////


void draw_track(GroverParticle* par, int colour = Black)
{
  float xpm[1], ypm[1];
  ismk(3);
  ismksc(0.5);
  ispmci(colour);
  isplci(colour);
    
  xpm[0] = par->BeginX();
  ypm[0] = par->BeginY();
  ipm(1,xpm,ypm);
  float xm = xpm[0];
  float ym = ypm[0];
  float dx = sqrt(par->BeginCov(0));
  float dy = sqrt(par->BeginCov(2));
  igarc(xm, ym, 0.5*(dx+dy),0.5*(dx+dy), 0.0, 0.0);

  iuwk(1,1); // update canvas
}


////////////////////////////////////////////////////////////////////////


void draw_vertex(GroverVertex* vertex, int colour = Red)
{
  float xpm[1], ypm[1];
  float xpl[5], ypl[5];

  ismk(30); 
  ismksc(3.0); 
  ispmci(colour); 
  isplci(colour);

  xpm[0]=vertex->X();
  ypm[0]=vertex->Y();
  ipm(1,xpm,ypm);
  
  float xv = vertex->X();
  float dx = 3.5*sqrt(vertex->Cov(0));
  float yv = vertex->Y();
  float dy = 3.5*sqrt(vertex->Cov(2));
  
  xpl[0] = xv-dx;
  ypl[0] = yv-dy;
  xpl[1] = xv+dx;
  ypl[1] = yv-dy;
  xpl[2] = xv+dx;
  ypl[2] = yv+dy;
  xpl[3] = xv-dx;
  ypl[3] = yv+dy;
  xpl[4] = xv-dx;
  ypl[4] = yv-dy;
  ipl(5,xpl,ypl);
  iuwk(1,1); // update canvas
}


/////////////////////////////////////////////////////////////////////////

