*CMZ :  1.00/00 03/01/95  12.31.48  by  Dominik Ressing
*CMZ :  0.01/02 26/08/94  12.00.22  by  Dominik Ressing
*-- Author :    Dominik Ressing   26/07/94
************************************************************************
*
* L1JOBEX
* written by Dominik Ressing, 26/07/94 17.52.28
*
* This routines action:
* returns pointer to the next to execute job or 0
* IF the busy flag is larger then the execution time of the job - the
*    job is requeued with the time of the PRCs reenable time. And the
*    next JOB tried.
* ELSE L1RUNcutime is set to the first complete cycle after the last PRCs
*    reenable.
* ENDIF
* The input FIFO of the PRC is handled accordingly.
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* NextJob     out         pointer to next to execute JOB
* ierr        out         error flag
*
************************************************************************
c
      SUBROUTINE L1JOBEX(NextJob,ierr)
c
      IMPLICIT NONE
c
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1runpa.fi"
#include "l1si/l1jobdf.fi"
#include "l1si/l1mesdf.fi"
#include "l1si/l1prcdf.fi"
c
      INTEGER ierr,NextJob,MessPo,idiff,Nadd,iFlag,
     &        iPrcPo,iType,iwait,itime,Nfif,Ncyc
c
      ierr = 0
c
 100  CONTINUE ! try again if we had to requeue job
c
      NextJob  = L1JOBstore(L1JOBsused)           ! next job to execute
      IF ( NextJob.eq.0 ) RETURN                  ! no further jobs
      L1RUNcutime = L1JOBstore(NextJob+L1JOBtime) ! earliest execution time for next job
      iFlag = L1JOBstore(NextJob+L1JOBflag)       ! job flag
c
      IF ( iFlag.eq.L1JOBFLAGinva ) THEN          ! job valid ???
         WRITE (L1LUNprn,*)
     &        'L1JOBEX==> Job invalid : FATAL',NextJob
         ierr = 1
         RETURN
      END IF
c
      MessPo = L1JOBstore(NextJob+L1JOBmess)
      iPrcPo = L1MEstor(MessPo+L1MEnextPC)        ! pointer to next processor
      IF ( iPrcPo.le.0 ) THEN                     ! fatal error !!!!!!!
         WRITE (L1LUNprn,*)
     &        'L1JOBEX==> PRC pointer zero : FATAL'
         ierr = 1
         RETURN
      END IF
c
      Nfif  = L1PCstor(iPrcPo+L1PCnfif)      ! number of messages in PRCs fifo
c
      iType = L1PCstor(iPrcPo+L1PCtype)      ! type of processor
      iwait = L1PCstor(iPrcPo+L1PCbusy)      ! re-enable-time of processor
      idiff = iwait - L1RUNcutime            !
      IF ( idiff.gt.0 ) THEN                 ! if prc closed -> requeue
         CALL l1jobdi(NextJob)               ! disconnect Job from stack
         CALL l1jobrc(iwait,NextJob)         ! reconnect Job at time iwait
         IF ( iFlag.eq.L1JOBFLAGnorm ) THEN  ! job not in FIFO
            Nfif = Nfif + 1                                ! increment fifo pointer
            L1PCstor(iPrcPo+L1PCnfif) = Nfif               ! write message into FIFO
            L1JOBstore(NextJob+L1JOBflag) = L1JOBFLAGfifo  ! flag job as in FIFO
            IF ( Nfif.ge.L1PCmaxfif.and.                   ! FIFO full ?
     &           L1PARprnlev.ge.1) THEN
               WRITE (L1LUNprn,*)                             ! no problem but warning
     &              'L1JOBEX==> FIFO overflow N/type/num',
     &              Nfif,
     &              L1PCstor(iPrcPo+L1PCtype),
     &              L1PCstor(iPrcPo+L1PCnum)
            END IF
            IF (L1PARprnlev.ge.1) THEN
               WRITE (L1LUNprn,898) NextJob,iPrcPo,Nfif
 898           FORMAT(1x,'L1JOBEX==> job ',i5,' into   FIFO PRC ',i5,
     &              ' entries ',i3)
            END IF
         END IF
         GO TO 100                           ! try next job
      ELSE
         Ncyc = int(-1*idiff/L1PCcyc(iType)) ! cycles since enable
         Nadd = Ncyc*L1PCcyc(iType)+idiff    ! time till next cycles end
         IF ( Nadd.ne.0 ) Nadd = Nadd + L1PCcyc(iType)
         L1RUNcutime = L1RUNcutime + Nadd
         IF ( iFlag.eq.L1JOBFLAGfifo ) THEN  ! was Job in FIFO ??
            L1PCstor(iPrcPo+L1PCnfif)     = Nfif-1         ! read message from fifo
            L1JOBstore(NextJob+L1JOBflag) = L1JOBFLAGnorm  ! remove FIFO flag
            IF (L1PARprnlev.ge.1) THEN
               WRITE (L1LUNprn,899) NextJob,iPrcPo,Nfif
 899           FORMAT(1x,'L1JOBEX==> job ',i5,' out of FIFO PRC ',i5,
     &              ' entries ',i3)
            END IF
         END IF
      END IF
c
      IF (L1PARprnlev.ge.1) THEN
         WRITE (L1LUNprn,900) NextJob,L1RUNcutime,MessPo
  900    FORMAT(1x,'L1JOBEX==> execute job ',i5,' at time ',i5,
     &            ' with message ',i5)
      END IF
c
      RETURN
c
      END
