// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include "arte/arte.hh"
#include "rang/Node.hh"
#include "rang/Hit.hh"
#include "rang/HitUnsigned.hh"
#include "rang/Geom.hh"
#include "rang/StateP.hh"
#include "rang/StatePI.hh"
#include "rang/StateL.hh"
#include "rang/StateLI.hh"
//#include "rang/zfconst.hh"
#include <math.h>

//-- Constructor
rangerHitUnsigned::rangerHitUnsigned() {};
rangerHitUnsigned::rangerHitUnsigned(ArtePointer<RHIT> rhit,rangerFitMode fm) 
: rangerHit(rhit,fm) {
  m = new HepVector(1);
  (*m)(1) = fabs(rhit->hitb->q);
  //(*m)(1) = LRSign * (fabs(rhit->hitb->q) - zfconst_.dOff);
  V = new HepSymMatrix(1);
  (*V)(1,1) = (rhit->hitb->r) * (rhit->hitb->r);
//  cout << "This is the rangerHitUnsigned constructor" << endl;
};

rangerHitUnsigned::rangerHitUnsigned(ArtePointer<HITB> hb,rangerFitMode fm) 
: rangerHit(hb,fm) {
  m = new HepVector(1);
  (*m)(1) = fabs(hb->q);
  //(*m)(1) = LRSign * (fabs(rhit->hitb->q) - zfconst_.dOff);
  V = new HepSymMatrix(1);
  (*V)(1,1) = (hb->r) * (hb->r);
//  cout << "This is the rangerHitUnsigned constructor" << endl;
};

rangerHitUnsigned::~rangerHitUnsigned() {
  delete m;
  delete V;
//  cout << "The HitUnsigned destructor deletes m and V" << endl;
}

void rangerHitUnsigned::projectP(HepVector& resid,
			     HepMatrix& HH,rangerStateP* s) {

#ifdef FULL
  cout << "This is rangerHitUnsigned::projectP" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();
  double tanThetaU = s->tx() * cosTot - s->ty() * sinTot;
  double cosThetaU = 1 / sqrt( 1 + tanThetaU * tanThetaU);
  double deltaU = s->x() * cosTot - s->y() * sinTot - getHitb()->x;
  double Hx = fabs(deltaU * cosThetaU);
  int S = (deltaU >=0)? 1 : -1;  // actual LR sign
  resid(1) = (*m)(1) - Hx;

#ifdef FULL
  cout << "rangerHitUnsigned::projectP: the stereo angle is " 
       << gede->sin << endl;
  cout << "rangerHitUnsigned::projectP: the residual is " << deltaU <<
       " error " << sqrt((*V)(1,1)) << endl;
#endif

  HH(1,1) =   S * cosTot * cosThetaU;
  HH(1,2) = - S * sinTot * cosThetaU;
  double ct3= cosThetaU * cosThetaU * cosThetaU;
  HH(1,3) = - S * deltaU * tanThetaU * ct3 * cosTot;
  HH(1,4) =   S * deltaU * tanThetaU * ct3 * sinTot;
  HH(1,5) = 0;

}

void rangerHitUnsigned::projectPI(HepVector& resid,
			      HepMatrix& HH,rangerStatePI* s) {

#ifdef FULL
  cout << "This is rangerHitUnsigned::projectPI" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();
  double tanThetaU = s->tx() * cosTot - s->ty() * sinTot;
  double cosThetaU = 1 / sqrt( 1 + tanThetaU * tanThetaU);
  double deltaU = s->x() * cosTot - s->y() * sinTot - getHitb()->x;
  // assume that the drift velocity is 0.01 cm/ns
  double Hx = fabs(deltaU * cosThetaU) - 0.01 * s->I();
  int S = (deltaU >=0)? 1 : -1;  // actual LR sign
  resid(1) = (*m)(1) - Hx;

#ifdef FULL
  cout << "rangerHitUnsigned::projectP: the stereo angle is " 
       << gede->sin << endl;
  cout << "rangerHitUnsigned::projectP: the residual is " << deltaU <<
       " error " << sqrt((*V)(1,1)) << endl;
#endif

  HH(1,1) =   S * cosTot * cosThetaU;
  HH(1,2) = - S * sinTot * cosThetaU;
  double ct3= cosThetaU * cosThetaU * cosThetaU;
  HH(1,3) = - S * deltaU * tanThetaU * ct3 * cosTot;
  HH(1,4) =   S * deltaU * tanThetaU * ct3 * sinTot;
  HH(1,5) = 0;
  // assume that the drift velocity is 0.01 cm/ns
  HH(1,6) = -0.01;
}


void rangerHitUnsigned::projectL(HepVector& resid,
			     HepMatrix& HH,rangerStateL* s) {

#ifdef FULL
  cout << "This is rangerHitUnsigned::projectL" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();
  double tanThetaU = s->tx() * cosTot - s->ty() * sinTot;
  double cosThetaU = 1 / sqrt( 1 + tanThetaU * tanThetaU);
  double deltaU = s->x() * cosTot - s->y() * sinTot - getHitb()->x;
  double Hx = fabs(deltaU * cosThetaU);
  int S = (deltaU >=0)? 1 : -1;  // actual LR sign
  resid(1) = (*m)(1) - Hx;

#ifdef FULL
  cout << "rangerHitUnsigned::projectL: the stereo angle is " 
       << gede->sin << endl;
  cout << "rangerHitUnsigned::projectL: the residual is " << deltaU <<
       " error " << sqrt((*V)(1,1)) << endl;
#endif

  HH(1,1) =   S * cosTot * cosThetaU;
  HH(1,2) = - S * sinTot * cosThetaU;
  double ct3= cosThetaU * cosThetaU * cosThetaU;
  HH(1,3) = - S * deltaU * tanThetaU * ct3 * cosTot;
  HH(1,4) =   S * deltaU * tanThetaU * ct3 * sinTot;

}


void rangerHitUnsigned::projectLI(HepVector& resid,
			      HepMatrix& HH,rangerStateLI* s) {

#ifdef FULL
  cout << "This is rangerHitUnsigned::projectLI" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();
  double tanThetaU = s->tx() * cosTot - s->ty() * sinTot;
  double cosThetaU = 1 / sqrt( 1 + tanThetaU * tanThetaU);
  double deltaU = s->x() * cosTot - s->y() * sinTot - getHitb()->x;
  // assume that the drift velocity is 0.01 cm/ns
  double Hx = fabs(deltaU * cosThetaU) - 0.01 * s->I();
  int S = (deltaU >=0)? 1 : -1;  // actual LR sign
  resid(1) = (*m)(1) - Hx;

#ifdef FULL
  cout << "rangerHitUnsigned::projectLI: the stereo angle is " 
       << gede->sin << endl;
  cout << "rangerHitUnsigned::projectLI: the residual is " << deltaU <<
       " error " << sqrt((*V)(1,1)) << endl;
#endif

  HH(1,1) =   S * cosTot * cosThetaU;
  HH(1,2) = - S * sinTot * cosThetaU;
  double ct3= cosThetaU * cosThetaU * cosThetaU;
  HH(1,3) = - S * deltaU * tanThetaU * ct3 * cosTot;
  HH(1,4) =   S * deltaU * tanThetaU * ct3 * sinTot;
  // assume that the drift velocity is 0.01 cm/ns
  HH(1,5) = -0.01;
}
