// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modifications:
// - Introduction of ::rotMatrix call     14-Aug-00 Wouter Hulsbergen
//

// class rangerGwal
// interface to GWAL table

#ifndef rangerGwal_HH
#define rangerGwal_HH

#ifndef __ARTE_HH__
#include "arte/arte.hh"
#endif
#ifndef rangerGeWa_HH
#include "rang/GeWa.hh"
#endif

class rangerGwal : public rangerGeWa {

public:
  rangerGwal();                      // default constructor
  rangerGwal(ArtePointer<GWAL> gw);  // constructor from GWAL wall 
  virtual ~rangerGwal();
  inline const int plNumber(){return -gwal.index();}; // mangled id
  inline const ArtePointer<GEDE> getGede() const {return 0;} //return gede
  inline const ArtePointer<GWAL> getGwal() const {return gwal;} //return gwal
  inline const int shp() const {return gwal->shp;};   // shape
  inline float ms() const {return gwal->ms;}   // mult scattering
  //inline const float cos(){return gwal->cos;}; // cos of stereo angle
  //inline const float sin(){return gwal->sin;}; // sin of stereo angle
  inline const float cos(){return 1;}; // cos of stereo angle
  inline const float sin(){return 0;}; // sin of stereo angle
  inline const float rot(int i){return gwal->rot[i-1];}; // rotation matrix
  const float* rotmatrix() const { return gwal->rot ; }
  inline const float xmin(){return gwal->xmin;}; // borders of total area
  inline const float xmax(){return gwal->xmax;};
  inline const float ymin(){return gwal->ymin;};
  inline const float ymax(){return gwal->ymax;};
  inline const float sxmin(){return gwal->xmin;};// borders of sensitive area
  inline const float sxmax(){return gwal->xmax;};
  inline const float symin(){return gwal->ymin;};
  inline const float symax(){return gwal->ymax;};
  inline const float zmin(){return gwal->zmin;};
  inline const float zmax(){return gwal->zmax;};
  inline const float xmad(){return gwal->xmad;};
  inline const float ymad(){return gwal->ymad;};
  inline const float zmad(){return gwal->zmad;};
  inline const float type(){return 0 ;};    // type (1=GEDE, 0=GWAL)
  inline const char gvol(const int i){return gwal->gvol[i];}

private:
  ArtePointer<GWAL> gwal;
};

#endif
