// 
//  Author:  Alexander Spiridonov              Created:  24.05.99
//
//  Email:   spiridon@ifh.de                   HERA-B Collaboration
// 
// rgextvdmuri.C
//                      match of VDS, MUON and RICH segments

#include <iostream.h>
#include <math.h>
#include <stdio.h>
#include "arte/arte.hh"
#include "rang/rgExtDef.hh"
#include "rang/zfparext.hh"

void rgextvdmuri(ArteTableIterator <RSEG>& segvd  // segment in VDS
	        ,ArteTableIterator <RSEG>& segmu // segment in MUON
	        ,ArteTableIterator <RSEG>& segri // segment in RICH
                ,float* p_vdmu                   // (x,y,tx,ty,Q/p) in MUON
	        ,int& OK_vdmuri                  // =1 for OK match,0 no match 
	        ,int& MC_vdmuri)                 // MC track ,=0 no MC track 
{
    
  cout << "rgextvdmuri : vds "  << segvd << " muon " << segmu 
       << " rich " << segri << endl;

  int CUT = 0;

#ifdef rgExtHist
  int id0 = zfparext_.id_hist; 
  int id;
  float w = 1.; float v;
  
#endif
  int isegvd,isegmu,imcvd,imcmu,imcri;
//  int isegri;
  float fracmcvd,fracmcmu;
//  float fracmcri;

  if(rgmc_())
    {
      isegvd=segvd.index();  
      tfremcse_(isegvd, imcvd, fracmcvd);
      isegmu=segmu.index();
      tfremcse_(isegmu, imcmu, fracmcmu);
//      isegri=segri.index();
      imcri = segri->fit;
//      cout << "rgextvdmuri : imcri,fit " << imcri << " " 
//      << segri->fit << endl;
    } 
  else
    {
      imcvd=-1; imcmu=0; imcri = -2;
    }

  cout << "rgextvdmuri : imcvd " << imcvd << " imcmu " << imcmu 
       << " imcri " << imcri << endl;
//                                                                 CUT 5

  if(fabsf(p_vdmu[3] - segri->tyf) > zfparext_.dmax_ty_vdmuri)
    {
      CUT =5;
      goto END998;
    }

#ifdef rgExtHist  
  hf1_(id=id0+207, v = p_vdmu[3] - segri->tyf, w);
  if(imcvd == imcmu && imcvd == imcri && imcvd != 0 )       // good pair
    {
      hf1_(id=id0+307, v = p_vdmu[3] - segri->tyf, w);
    }
#endif

//                                                                 CUT 6

  if(fabsf(p_vdmu[2] - segri->txf) > zfparext_.dmax_tx_vdmuri)
    {
      CUT =6;
      goto END998;
    }

#ifdef rgExtHist  
  hf1_(id=id0+208, v = p_vdmu[2] - segri->txf, w);
  if(imcvd == imcmu && imcvd == imcri && imcvd != 0 )       // good pair
    {
      hf1_(id=id0+308, v = p_vdmu[2] - segri->txf, w);
    }
#endif

  float Teta_rich;
  float beta,mass_mu;
  mass_mu = 0.105658389;
  beta = 1./sqrt(1. + mass_mu * mass_mu * p_vdmu[4] * p_vdmu[4]);  
//  Teta_rich = 1.4142136 * sqrt(1.-1./(zfparext_.refrac_rich * beta));
  Teta_rich = acos(1./(zfparext_.refrac_rich * beta));
  Teta_rich *= 1.008049;

  float mass;
  mass = fabsf(1./p_vdmu[4]) 
    * sqrt(1./(zfparext_.refrac_rich *zfparext_.refrac_rich
	       * (1. - segri->time *segri->time / 2.)) - 1.);

//  Teta_rich = 1.4142136 * sqrt(1.-1./zfparext_.refrac_rich
//	     *sqrt(1.+ 0.011163697 *(p_vdmu[4] * p_vdmu[4]));

  mass = segri->time / Teta_rich;

  cout << "Teta_rich " << Teta_rich << " Teta_rec " << segri->time 
       << " Mass " << mass << endl;

#ifdef rgExtHist  
  hf1_(id=id0+209, v = segri->time, w);
  if(imcvd == imcmu && imcvd == imcri && imcvd != 0 )       // good pair
    {
      hf1_(id=id0+309, v = segri->time, w);
    }
  else
    {
      hf1_(id=id0+409, v = segri->time, w);
    }

#endif

//                                                                 CUT 7

  if(fabsf(segri->time/Teta_rich - 1.) > zfparext_.dmax_teta_mu)
    {
      CUT =6;
      goto END998;
    }

#ifdef rgExtHist  
  hf1_(id=id0+210, v = segri->time/Teta_rich - 1., w);
  if(imcvd == imcmu && imcvd == imcri && imcvd != 0 )       // good pair
    {
      hf1_(id=id0+310, v = segri->time/Teta_rich - 1., w);
    }
  else
    {
      hf1_(id=id0+410, v = segri->time/Teta_rich - 1., w);
    }

#endif

  if(imcvd == imcmu && imcvd == imcri && imcvd != 0 )       // good pair
    {
      MC_vdmuri = imcvd;
    }
  else
    {
      MC_vdmuri = 0;
    }

  OK_vdmuri = 1;
  return;

 END998:

#ifdef rgExtHist
  cout << " rgextvdmuri :                                     vdmuri CUT= "
       << CUT << endl;
#endif

  OK_vdmuri = 0;
  return;

}   //   end of rgextvdmu_()

//  cout << "rgextvdmu : isegvd " << isegvd 
//       << " imcvd " << imcvd << " fracmcvd " << fracmcvd << endl;
 
//  cout << "rgextvdmu : isegmu " << isegmu 
//       << " imcmu " << imcmu << " fracmcmu " << fracmcmu << endl;
