// 
//  Author:  A.Spiridonov                      Created:  17-May_2000
//  Email:   spiridon@ifh.de                   HERA-B Collaboration 
//  
#include <iostream.h>
#include <math.h>
#include "arte/arte.hh"
#include "rang/Fit.hh"
#include "rang/seg2track.hh"
#include "rang/tfmt2seg.hh"
#include "rang/tffhist.hh"
#include "rang/Geom.hh"
#include "rang/rginit.hh"

// rgfitpar_
// Fortran interface to int rangerFit(ArtePointer<RSEG> rseg,
// rangerParticleType pType).
// Applies ranger fit to a RSEG entry.
// Output is written back to the corresponding RHIT/RPNT entities
// and in array par[20] ( 1-5 parameters at first point, 6-20 covariance matrix).
// Second parameter specifies if electron radiation correction is wanted.
// Self-initialized.
// Tailored version of rgfit.C

extern "C" int rgfitpar_(
			 const int& rs,  // row number of RSEG to be fitted 
			 const int &elec_corr,  // electron correction mode
			                       //    1 - on
			                       //    0 - off      
			 float* par            // parameters at first point
			                       // xf,yf,txf,tyf,qf,covf[15] 
			 )
{  
  
  //  cout << " rgfitpar.C : rs " << rs << endl;
  ArtePointer<RSEG> rseg(rs);
  float parf[20],pare[20];
  // store parameters of segment
  parf[0] = rseg->xf;
  parf[1] = rseg->yf;
  parf[2] = rseg->txf;
  parf[3] = rseg->tyf;
  parf[4] = rseg->pf;
  for (int i = 0; i < 15; ++i) {parf[5+i] = rseg->cvf[i];}  
  int par_fit = rseg->fit;
  int par_err = rseg->err;
  //  cout << " cvf " << rseg->cvf[0] << " " << rseg->cvf[2] << " " << rseg->cvf[5]
  //     << " " << rseg->cvf[9] << " " << rseg->cvf[14] << endl;
  pare[0] = rseg->xe;
  pare[1] = rseg->ye;
  pare[2] = rseg->txe;
  pare[3] = rseg->tye;
  pare[4] = rseg->pe;
  for (int i = 0; i < 15; ++i) {pare[5+i] = rseg->cve[i];}  

  if (ArtePointer<RSEG>::nrow() <= 0) return 0;
  
  // geometry initialization
  rginit_();
  rangerGeom::condUpdate();
  
  //  ArtePointer<RSEG> rseg(rs);
  int fitStatus;
  if(rseg->ze < 210. )
    {
      fitStatus = (elec_corr) ? rangerFitV(rseg,0,ELECTRON) : rangerFitV(rseg,0);
      //      fitStatus = (elec_corr) ? rangerFit(rseg,ELECTRON) : rangerFit(rseg);
    }
  else
    {
      fitStatus = (elec_corr) ? rangerFit(rseg,ELECTRON) : rangerFit(rseg);
    }
  // return new parameters
  par[0] = rseg->xf;
  par[1] = rseg->yf;
  par[2] = rseg->txf;
  par[3] = rseg->tyf;
  par[4] = rseg->pf;
  for (int i = 0; i < 15; ++i) {par[5+i] = rseg->cvf[i];}  
  // restore parametrs of segment
  rseg->xf = parf[0];
  rseg->yf = parf[1];
  rseg->txf = parf[2];
  rseg->tyf = parf[3];
  rseg->pf = parf[4];
  for (int i = 0; i < 15; ++i) {rseg->cvf[i] = parf[5+i];}  
  rseg->xe = pare[0];
  rseg->ye = pare[1];
  rseg->txe = pare[2];
  rseg->tye = pare[3];
  rseg->pe = pare[4];
  for (int i = 0; i < 15; ++i) {rseg->cve[i] = pare[5+i];}  
  rseg->fit = par_fit;
  rseg->err = par_err;

  return fitStatus;
}
