// 
//  Author:  Rainer Mankel                     Created:  22-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// rgxseg_
// Calculates the x position of the stored horizontal track candidate
// in a given domain at the given z. This is done by extrapolating
// the track position/direction stored for this domain.

// If the information is unavailable, zero is returned.

#include "rang/Domain.hh"
#include "rang/XWin.hh"

extern "C" {
  float rgxseg_(const float& z,rangerDomain* const & dm) {
    
    if (dm) {
      if (! dm->xwin()) {
	cout << "rgxseg: no xwin defined" << endl;
	return 0;
      }
      if (! dm->xwin()->filled()) {
	cout << "rgxseg: xwin is not filled! dom=" << dm << endl;
	return 0;
      }
//       cout << "rgxseg: xwin=" << dm->xwin()
// 	   << " x=" << dm->xwin()->x()
// 	   << " z=" << dm->xwin()->z()
// 	   << " tx=" << dm->xwin()->tx()
// 	   << " znom=" << z << endl;
      return (dm->xwin()->x() + (z - dm->xwin()->z()) * dm->xwin()->tx()); 
    }
    else {
      cout << " () rgxseg: domain unavailable" << endl;
      return 0;
    }
  }
}
