
// ****************************************************************************
//
// Routine:      rc99Aevnt
// Author:       T.Oest/D.Samtleben
//
// Description:  event reconstruction
//
// Modification:
// matching added                                                   03/06/99 TO
// vertexing added                                                  06/09/99 TO
// particle ID added                                                16/09/99 TO
// event classification added, select decay candidates              29/10/99 TO
// track points RPNT filled                                         07/01/00 TO
// fill EVHI bank                                                   09/02/00 TO
// introduce MODULE                                                 11/04/00 AG
// introduce timers                                                 17/05/00 AG
// times to globals                                                 18/05/00 AG
// initialize timers to 0                                           07/06/00 AG
// remove unused timers again                                       07/06/00 AG
// move event classification to rcevnt                              14/06/00 AG
// polishing                                                        16/06/00 AG
// removed the filling of track points RPNT (now done in ranger)    21/06/00 DS
// check for rhp dispatch messages (DQM) frequently                 30/06/00 AG
// check for incomming rpm messages                                 10/07/00 AG
// rhp handling is done somewhere else                              19/07/00 AG
// call to rcMoment (determination of OTR momentum) included        23/07/00 DS
// track points RPNT filled (if not already done by ranger)         23/07/00 DS
// added select flags                                               25/07/00 AG
// introduced module selection                                      26/07/00 AG
// initialized timers correctly                                     27/07/00 AG
// added ARTE_mark()                                                04/08/00 AG
// polished code                                                    17/08/00 AG
//
// ****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>

// ARTE includes
#include "arte/arte.hh"
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
#include "farm/farm.hh"
#include "reco/rc99Areco.hh"

// globals
static int firstcall = 1;

float time_rc99AStandalone  = 0;
float time_rc99Match        = 0;
float time_rc99SelectTracks = 0;
float time_rc99Arefit       = 0;
float time_rcSetTrackPoints = 0;
float time_rc99AParticleID  = 0;
float time_rc99Avertex      = 0;
float time_rcSelectDecays   = 0;

int selStandAlone   = 1;
int selMatch        = 1;
int selRefit        = 1;
int selParticleID   = 1;
int selVertex       = 1;
int selSelectDecays = 1;

// externals
extern int FATIME;
extern int FADEB;

// prototypes
int  rc99Arefit();
int  rc99SelectTracks();
int  rcSelectDecays();
int  rcSetTrackPoints();
int  rcMoment();
void rc99Avertex();
void rcfillEVHI();
int  rcModSel();

//*****************************************************************
// function   no action for MC
//*****************************************************************
void
rc99Aevnt()
{
  // mark
  ARTE_mark( "rc99Aevnt 0", 0 );

  //==================================================================
  // 1. initialization, return for MC
  //==================================================================
  ArtePointer<EVHD> header(1);
  
  if (ArteTable<MTRA>::begin() != ArteTable<MTRA>::end()){
    if (header->exp > 0 )         header->exp = -1;
  }

  // timers
  static Chrono *cron_rc99AStandalone  = 0;
  static Chrono *cron_rc99Match        = 0;
  static Chrono *cron_rc99SelectTracks = 0;
  static Chrono *cron_rc99Arefit       = 0;
  static Chrono *cron_rcSetTrackPoints = 0;
  static Chrono *cron_rc99AParticleID  = 0;
  static Chrono *cron_rc99Avertex      = 0;
  static Chrono *cron_rcSelectDecays   = 0;

  // inits
  if (firstcall == 1 ) {
    firstcall = 0;
    
    // rc99Ainit
    ARTE_mark( "rc99Ainit", 0 );
    rc99Ainit();
    
    // timers
    ChronoManager CM;
    CM.instance();

    cron_rc99AStandalone  = CM.createChrono( "rc99AStandalone" );
    cron_rc99Match        = CM.createChrono( "rc99Match" );
    cron_rc99SelectTracks = CM.createChrono( "rc99SelectTracks" );
    cron_rc99Arefit       = CM.createChrono( "rc99Arefit" );
    cron_rcSetTrackPoints = CM.createChrono( "rcSetTrackPoints" );
    cron_rc99AParticleID  = CM.createChrono( "rc99AParticleID" );
    cron_rc99Avertex      = CM.createChrono( "rc99Avertex" );
    cron_rcSelectDecays   = CM.createChrono( "rcSelectDecays" );
  }

  // initialization
  int rcStandAloneFlag = 0;
  int MatchingFlag     = 0;
  int RTRAFlag         = 0;
  int refit            = 0;
  int ParticleID       = 0;
  int SelectDecays     = 0;

  //==================================================================
  // stand alone reconstruction,
  // momentum determination of OTR segments
  //==================================================================
  time_rc99AStandalone = 0;
  if( selStandAlone ){
    ARTE_mark( "rc99AStandAlone", 0 );
    cron_rc99AStandalone->start();
    rcStandAloneFlag = rc99AStandAlone();
    
    int momflag=rcMoment();
    
    cron_rc99AStandalone->stop();
    time_rc99AStandalone = cron_rc99AStandalone->lastTime();
  }

  //==================================================================
  // matching, track selection
  //==================================================================
  time_rc99Match        = 0;
  time_rc99SelectTracks = 0;
  if( selMatch ){
    ARTE_mark( "rc99Match", 0 );
    cron_rc99Match->start();
    MatchingFlag = rc99Match();
    cron_rc99Match->stop();
    time_rc99Match = cron_rc99Match->lastTime();
  
    ARTE_mark( "rc99SelectTracks", 0 );
    cron_rc99SelectTracks->start();
    RTRAFlag = rc99SelectTracks();
    cron_rc99SelectTracks->stop();
    time_rc99SelectTracks = cron_rc99SelectTracks->lastTime();
  }

  //==================================================================
  // refit of tracks
  //==================================================================
  time_rc99Arefit       = 0;
  time_rcSetTrackPoints = 0;
  if( selRefit ){
    ARTE_mark( "rc99Arefit", 0 );
    cron_rc99Arefit->start();
    refit = rc99Arefit();
    cron_rc99Arefit->stop();
    time_rc99Arefit = cron_rc99Arefit->lastTime();
    
    ARTE_mark( "rcSetTrackPoints", 0 );
    cron_rcSetTrackPoints->start();
    int RPNTFlag = rcSetTrackPoints();
    cron_rcSetTrackPoints->stop();
    time_rcSetTrackPoints = cron_rcSetTrackPoints->lastTime();
  }

  //==================================================================
  // Particle ID
  //==================================================================
  time_rc99AParticleID = 0;
  if( selParticleID ){
    ARTE_mark( "rc99AParticleID", 0 );
    cron_rc99AParticleID->start();
    ParticleID = rc99AParticleID();
    cron_rc99AParticleID->stop();
    time_rc99AParticleID = cron_rc99AParticleID->lastTime();  
  }

  //==================================================================
  // vertexing
  //==================================================================
  time_rc99Avertex = 0;
  if( selVertex ){
    ARTE_mark( "rc99Avertex", 0 );
    cron_rc99Avertex->start();
    rc99Avertex();
    cron_rc99Avertex->stop();
    time_rc99Avertex = cron_rc99Avertex->lastTime();
  }

  //==================================================================
  // reconstruction of decays
  //==================================================================
  time_rcSelectDecays = 0;
  if( selSelectDecays ){
    ARTE_mark( "rcSelectDecays", 0 );
    cron_rcSelectDecays->start();
    SelectDecays = rcSelectDecays();
    cron_rcSelectDecays->stop();
    time_rcSelectDecays = cron_rcSelectDecays->lastTime();
  }

  //==================================================================
  // fill EVHI bank
  //==================================================================
  ARTE_mark( "rcfillEVHI", 0 );
  rcfillEVHI();
  
  //==================================================================
  // print times
  //==================================================================
  if( FATIME ){

    float sum = 
      time_rc99AStandalone  +
      time_rc99Match        +
      time_rc99SelectTracks +
      time_rc99Arefit       +
      time_rcSetTrackPoints +
      time_rc99AParticleID  +
      time_rc99Avertex      +
      time_rcSelectDecays;

    cout << " RECO: "
	 << "s-a="  << time_rc99AStandalone  << " "
	 << "mtch=" << time_rc99Match        << " "
	 << "trks=" << time_rc99SelectTracks << " "
	 << "ref="  << time_rc99Arefit       << " "
	 << "pts="  << time_rcSetTrackPoints << " "
	 << "pid="  << time_rc99AParticleID  << " "
	 << "vtx="  << time_rc99Avertex      << " "
	 << "decs=" << time_rcSelectDecays   << " "
	 << "sum="  << sum                   << endl;
  }
  
  // mark
  ARTE_mark( "rc99Aevnt 1", 0 ); 
}
