
// *****************************************************************************
//
// source:    rc99Akaclass.C
//
// type:      source code
//
// created:   20 Jul 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rc99Akaclass
//
// called by: 
//
// purpose:   handle kuip commands /RECO/CLASS
//
// changes:
// 20 Jul 2000 (AG) creation
// 25 Jul 2000 (AG) polished code
// 27 Jul 2000 (AG) improved printout
// 02 Aug 2000 (AG) introduced ECLASS_DVTX, ECLASS_SLTHPT
// 03 Aug 2000 (AG) improved kumac handling
// 03 Aug 2000 (AG) SLTHPT -> SLTCUT
// 04 Aug 2000 (AG) removed bug
// 16 Aug 2000 (AG) introdcued SLTYPS
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "reco/eclass.hh"

// globals
int classJPSIEE = 0; // bit  1
int classJPSIMM = 0; // bit  2
int classFCNC   = 0; // bit  3
int classCAL1HT = 0; // bit  4
int classVZERO  = 0; // bit  5
int classVZERO2 = 0; // bit  6
int classDVTX   = 0; // bit  7
int classSLTYPS = 0; // bit  8
int classSLTCUT = 0; // bit  9
int classSLTEEB = 0; // bit 10
int classSLTEE  = 0; // bit 11
int classSLTMM  = 0; // bit 12
int classSLTDV  = 0; // bit 13
int classTLT    = 0; // bit 14
int classFLTTRG = 1; // bit 15
int classRNDTRG = 1; // bit 16

// externals
extern int FADEB;

// prototypes
extern "C" void kugetc_( char*, int *, int );

//
// function code
//
extern "C" void
kaclass_()
{
  ARTE_mark( "kaclass_ 0", 0 );

  //
  // read in class
  //
  int len  = 0;     
  int len1 = 6;
  char input[7];
  memset( input, '\0', 7 ); 
  kugetc_( input, &len, len1 );
  input[len] = '\0';
  char *cl = (char *) malloc( len+1 );
  memset( cl, '\0', len+1 );
  strncpy( cl, input, len );
  for( int i=0; i<len; i++ ) cl[i] = toupper( cl[i] );
  if( strcmp( cl, "ALL" )    && //  all
      strcmp( cl, "JPSIEE" ) && //  1
      strcmp( cl, "JPSIMM" ) && //  2
      strcmp( cl, "CAL1HT" ) && //  3
      strcmp( cl, "FCNC"   ) && //  4
      strcmp( cl, "VZERO"  ) && //  5
      strcmp( cl, "VZERO2" ) && //  6
      strcmp( cl, "DVTX"   ) && //  7
      strcmp( cl, "SLTYPS" ) && //  8
      strcmp( cl, "SLTCUT" ) && //  9
      strcmp( cl, "SLTEEB" ) && // 10
      strcmp( cl, "SLTEE"  ) && // 11
      strcmp( cl, "SLTMM"  ) && // 12
      strcmp( cl, "SLTDV"  ) && // 13
      strcmp( cl, "TLT"    ) && // 14
      strcmp( cl, "FLTTRG" ) && // 15
      strcmp( cl, "RNDTRG" )    // 16
      ){ 
    cout << "RECON/CLASS: class=" << cl << " UNKOWN!" << endl;
    return;
  }
  
  //
  // read in status
  //
  len1 = 3;
  memset( input, '\0', 7 );
  kugetc_( input, &len, len1 );
  input[len] = '\0';
  char *status  = (char *) malloc( len+1 );
  memset( status, '\0', len+1 );
  strncpy( status, input, len );
  if( len>0 ) for( int i=0; i<len; i++ ) status[i] = toupper( status[i] );
  if( strcmp( status, "ON"  ) &&
      strcmp( status, "OFF" )    ){
    cout << "RECON/CLASS: status=" << status << " UNKOWN!" << endl;
    return;
  }
  
  //
  // print
  //
  if( FADEB ) cout << "RECON/CLASS: " << cl << " " << status << endl;

  //
  // set classification flags
  //
  if( !strcmp( status, "ON" ) ){

    if( !strcmp( cl, "ALL"    ) ){
      classJPSIEE = 1; //  1
      classJPSIMM = 1; //  2
      classFCNC   = 1; //  3
      classCAL1HT = 1; //  4
      classVZERO  = 1; //  5
      classVZERO2 = 1; //  6
      classDVTX   = 1; //  7
      classSLTYPS = 1; //  8
      classSLTCUT = 1; //  9
      classSLTEEB = 1; // 10
      classSLTEE  = 1; // 11
      classSLTMM  = 1; // 12
      classSLTDV  = 1; // 13
      classTLT    = 1; // 14
      classFLTTRG = 1; // 15
      classRNDTRG = 1; // 16
    }

    if( !strcmp( cl, "JPSIEE" ) ) classJPSIEE = 1; //  1
    if( !strcmp( cl, "JPSIMM" ) ) classJPSIMM = 1; //  2
    if( !strcmp( cl, "FCNC"   ) ) classFCNC   = 1; //  3
    if( !strcmp( cl, "CAL1HT" ) ) classCAL1HT = 1; //  4
    if( !strcmp( cl, "VZERO"  ) ) classVZERO  = 1; //  5
    if( !strcmp( cl, "VZERO2" ) ) classVZERO2 = 1; //  6
    if( !strcmp( cl, "DVTX"   ) ) classDVTX   = 1; //  7
    if( !strcmp( cl, "SLTYPS" ) ) classSLTYPS = 1; //  8
    if( !strcmp( cl, "SLTCUT" ) ) classSLTCUT = 1; //  9
    if( !strcmp( cl, "SLTEEB" ) ) classSLTEEB = 1; // 10
    if( !strcmp( cl, "SLTEE"  ) ) classSLTEE  = 1; // 11
    if( !strcmp( cl, "SLTMM"  ) ) classSLTMM  = 1; // 12
    if( !strcmp( cl, "SLTDV"  ) ) classSLTDV  = 1; // 13
    if( !strcmp( cl, "TLT"    ) ) classTLT    = 1; // 14
    if( !strcmp( cl, "FLTTRG" ) ) classFLTTRG = 1; // 15
    if( !strcmp( cl, "RNDTRG" ) ) classRNDTRG = 1; // 16
  }

  if( !strcmp( status, "OFF" ) ){
    if( !strcmp( cl, "ALL"    ) ){
      classJPSIEE = 0; //  1
      classJPSIMM = 0; //  2
      classFCNC   = 0; //  3
      classCAL1HT = 0; //  4
      classVZERO  = 0; //  5
      classVZERO2 = 0; //  6
      classDVTX   = 0; //  7
      classSLTYPS = 0; //  8
      classSLTCUT = 0; //  9
      classSLTEEB = 0; // 10
      classSLTEE  = 0; // 11
      classSLTMM  = 0; // 12
      classSLTDV  = 0; // 13
      classTLT    = 0; // 14
      classFLTTRG = 0; // 15
      classRNDTRG = 0; // 16
    }
    if( !strcmp( cl, "JPSIEE" ) ) classJPSIEE = 0; //  1
    if( !strcmp( cl, "JPSIMM" ) ) classJPSIMM = 0; //  2
    if( !strcmp( cl, "FCNC"   ) ) classFCNC   = 0; //  3
    if( !strcmp( cl, "CAL1HT" ) ) classCAL1HT = 0; //  4
    if( !strcmp( cl, "VZERO"  ) ) classVZERO  = 0; //  5
    if( !strcmp( cl, "VZERO2" ) ) classVZERO2 = 0; //  6
    if( !strcmp( cl, "DVTX"   ) ) classDVTX   = 0; //  7
    if( !strcmp( cl, "SLTYPS" ) ) classSLTYPS = 0; //  8
    if( !strcmp( cl, "SLTCUT" ) ) classSLTCUT = 0; //  9
    if( !strcmp( cl, "SLTEEB" ) ) classSLTEEB = 0; // 10
    if( !strcmp( cl, "SLTEE"  ) ) classSLTEE  = 0; // 11
    if( !strcmp( cl, "SLTMM"  ) ) classSLTMM  = 0; // 12
    if( !strcmp( cl, "SLTDV"  ) ) classSLTDV  = 0; // 13
    if( !strcmp( cl, "TLT"    ) ) classTLT    = 0; // 14
    if( !strcmp( cl, "FLTTRG" ) ) classFLTTRG = 0; // 15
    if( !strcmp( cl, "RNDTRG" ) ) classRNDTRG = 0; // 16
  }

  //
  // print
  //
  cout << "RECON/CLASS: " << cl << " " << status << endl;
  if( FADEB ){
    cout << "RECON/CLASS: flags:"                  << endl;
    cout << "             JPSIEE= " << classJPSIEE << endl; //  1
    cout << "             JPSIMM= " << classJPSIMM << endl; //  2
    cout << "             FCNC=   " << classFCNC   << endl; //  3
    cout << "             CAL1HT= " << classCAL1HT << endl; //  4
    cout << "             VZERO=  " << classVZERO  << endl; //  5
    cout << "             VZERO2= " << classVZERO2 << endl; //  6
    cout << "             DVTX=   " << classDVTX   << endl; //  7
    cout << "             SLTYPS= " << classSLTYPS << endl; //  8
    cout << "             SLTCUT= " << classSLTCUT << endl; //  9
    cout << "             SLTEEB  " << classSLTEEB << endl; // 10
    cout << "             SLTEE=  " << classSLTEE  << endl; // 11
    cout << "             SLTMM=  " << classSLTMM  << endl; // 12
    cout << "             SLTDV=  " << classSLTDV  << endl; // 13
    cout << "             TLT=    " << classTLT    << endl; // 14
    cout << "             FLTTRG= " << classFLTTRG << endl; // 15
    cout << "             RNDTRG= " << classRNDTRG << endl; // 16
  }

  ARTE_mark( "kaclass_ 1", 0 );
}
