
// *****************************************************************************
//
// source:    rcSelectVZERO.C
//
// type:      source code
//
// created:   19 Jul 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcSelectVZERO
//
// called by: rcClassVZERO
//
// purpose:   select events with V0 candidate
//
// hints:     rc = 0x0 : nothing
//                 0x1 : K0s    (Ignacio)
//                 0x2 : Lambda (Ignacio)
//
// changes:
// 19 Jul 2000 (AG) creation
// 21 Jul 2000 (AG) included code from Ignacio Arinyo (K0sSelect.C)
// 25 Jul 2000 (AG) corrected bug
// 26 Jul 2000 (AG) included Denis' selection
// 26 Jul 2000 (AG) move Denis' stuff to VZERO2
// 27 Jul 2000 (AG) improved printout
// 03 Aug 2000 (IA) bug removal
// 04 Aug 2000 (IA) another bug removal
// 05 Aug 2000 (AG) removed printout
// 01 Sep 2000 (IA) cuts modified: min distance to active wire to 5 cm
//                    max distance between tracks 0.05 cm
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <map.h>

// ARTE include
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "CLHEP/Hist/HBookTuple.h"
#include "CLHEP/Vector/ThreeVector.h"
#include "marple/marple.hh"
#include "arte/ArteUtil.h"
#include "arte/RSEG_cmp.hh"

// externals
extern int FADEB;

// prototypes
int trSelect( vector< ArtePointer<RTRA> > &rtra );

// 
// function code
//
float 
imass( ArtePointer<RTRA> track1,ArtePointer<RTRA> track2,
       float mass1, float mass2 )
{
  Hep3Vector p1(track1->txf,track1->tyf,1);
  p1*=1/sqrt(pow(track1->pf,2)*(pow(track1->txf,2)+pow(track1->tyf,2)+1));
  
  Hep3Vector p2(track2->txf,track2->tyf,1);
  p2*=1/sqrt(pow(track2->pf,2)*(pow(track2->txf,2)+pow(track2->tyf,2)+1));
  
  float imas=pow(sqrt(mass1*mass1+p1.mag2())+sqrt(mass2*mass2+p2.mag2()),2)-
    (p1+p2).mag2();
  
  //  cout<<p1<<1/track1->pf<<" "<<track1->txf<<" "<<track1->tyf<<" "
  //      <<p1.mag2()<<endl;
  //  cout<<p2<<(p1+p2).mag2()<<" "<<imas<<endl;
  if (imas>0){
    imas=sqrt(imas);
    return imas;
  }
  else return 0;
}

//
// function code
//
float 
imass( RcPoint rcp1,RcPoint rcp2,
       float mass1, float mass2)
{
  Hep3Vector p1(rcp1.tx(),rcp1.ty(),1);
  p1*=1./sqrt(pow(rcp1.pe(),2)*(pow(rcp1.tx(),2)+pow(rcp1.ty(),2)+1));
  
  Hep3Vector p2(rcp2.tx(),rcp2.ty(),1);
  p2*=1./sqrt(pow(rcp2.pe(),2)*(pow(rcp2.tx(),2)+pow(rcp2.ty(),2)+1));
  
  float imas=pow(sqrt(mass1*mass1+p1.mag2())+sqrt(mass2*mass2+p2.mag2()),2)-
    (p1+p2).mag2();
  
  if (imas>0){
    imas=sqrt(imas);
    return imas;
  }
  else return 0;
}

//
// function code
//
int
rcSelectVZERO()
{
  ARTE_mark( "rcSelectVZERO 0", 0 );
  
  //
  // Ignacio ******************************************************************
  //
  //find the active target with higher Z
  float MaxZtar=-10;
  for(ArteTable<GTAR>::iterator itgtar=ArteTable<GTAR>::begin();
      itgtar!=ArteTable<GTAR>::end();itgtar++){
    if(sqrt(pow((*itgtar)->x,2)+pow((*itgtar)->y,2))<1 &&
       (*itgtar)->z > MaxZtar) MaxZtar= (*itgtar)->z;
  }

  int k0s    = 0;
  int lambda = 0;

  float pimass=0.1395699;
  float kmass=0.493677;
  float pmass=0.938272;
  
  // Track selection
  vector< ArtePointer<RTRA> > VecRtra;
  int numtras= trSelect( VecRtra); 
    
  for(int i=0;i<int( VecRtra.size() );i++){
    
    if(fabs(1/VecRtra[i]->pf)<1 || fabs(1/VecRtra[i]->pf)>200) continue;
    //if(VecRtra[i]->zf >500) continue;
    
    for(int j=i+1;j<int(VecRtra.size());j++){

      if(fabs(1./VecRtra[j]->pf)<1 || fabs(1./VecRtra[j]->pf)>200) continue;


      //Find the point of closest approach of the two tracks, assuming
      //straight lines

      Hep3Vector p1(VecRtra[i]->xf,VecRtra[i]->yf,VecRtra[i]->zf);
      Hep3Vector p2(VecRtra[j]->xf,VecRtra[j]->yf,VecRtra[j]->zf);
      
      Hep3Vector d1(VecRtra[i]->txf,VecRtra[i]->tyf,1.);
      //      Hep3Vector d1(rsegVds1->txf,rsegVds1->tyf,1.);
      d1=d1.unit();
      
      Hep3Vector d2(VecRtra[j]->txf,VecRtra[j]->tyf,1.);
      //Hep3Vector d2(rsegVds2->txf,rsegVds2->tyf,1.);
      d2=d2.unit();
      
      float p1d1=p1*d1;
      float p1d2=p1*d2;
      float p2d1=p2*d1;
      float p2d2=p2*d2;
      float d1d2=d1*d2;
      
      float beta=(p1d2-p2d2+(p2d1-p1d1)*d1d2)/(1.-d1d2*d1d2);
      float alpha=p2d1-p1d1+beta*d1d2;
      
      float separation=(p1+alpha*d1-p2-beta*d2).mag();
      float zvertex=(p1+alpha*d1).z();
      
      	
      float im2pi=imass(VecRtra[i],VecRtra[j],pimass,pimass);
      //float impik=imass(VecRtra[i],VecRtra[j],pimass,kmass);
      //float imkpi=imass(*rtrait1,*rtrait2,kmass,pimass);
      //float im2k=imass(VecRtra[i],VecRtra[j],kmass,kmass);
      float impip=imass(VecRtra[i],VecRtra[j],pimass,pmass);
      float imppi=imass(VecRtra[i],VecRtra[j],pmass,pimass);
      
      
      if(separation<0.05 && zvertex>MaxZtar+5 && 
         VecRtra[i]->pf * VecRtra[j]->pf<0){
	//if(fabs(im2pi-0.4977)<0.1) arteutil_.eventout=1;
	//if(fabs(impip-1.115)<0.035) arteutil_.eventout=1;
	if(fabs(im2pi-0.4977)<0.05) k0s=1;
	if(fabs(impip-1.1156)<0.02 || fabs(imppi-1.1156)<0.02) lambda=2;
	
      }	
    }
  }

  // print
  if( FADEB ){
    cout << " SELECT: VZERO "
         << "k0s="    << k0s    << " "
         << "lambda=" << lambda << endl;
  }

  //
  // final return code
  //
  int rc = 0;

  // Ignacio
  if( k0s    ) rc |= 0x1; // K0s
  if( lambda ) rc |= 0x2; // Lambda

  // print
  if( FADEB ) cout << " SELECT: VZERO=" << rc << endl; 

  ARTE_mark( "rcSelectVZERO 1", 0 );
  return rc;
}


//
// function for track selection from Michael Schmelling
//
int 
trSelect( vector< ArtePointer<RTRA> > &rtra )
{ 
  //-------------------------------------------------------
  // return a vector with ArtePointers into RTRA
  //-------------------------------------------------------
  
  static int MINHITS_VDS  = 6;   // required minimum number of VDS hits
  static int MINHITS_MAIN = 20;  // required minimum number of Maintracker hits

  map< float, ArtePointer<RTRA> > mrtra;
  //------------------------------------
  
  // put all VDS-OTR tracks into a sorted map

  for(ArteTable<RTRA>::iterator itrtra = ArteTable<RTRA>::begin();
      itrtra != ArteTable<RTRA>::end(); ++itrtra) {
    if( (*itrtra)->fit !=  0 ) continue;
    int nvds  = (*itrtra)->hsi;
    int nmain = (*itrtra)->hou + (*itrtra)->hin;
    if( nvds  < MINHITS_VDS ) continue;
    if( nmain < MINHITS_MAIN) continue;
    float fom = float(nvds+nmain) + 1./(1.+(*itrtra)->chi);
    mrtra.insert(pair< float, ArtePointer<RTRA> >(-fom,*itrtra));
  }

  // copy the tracks into the return vector, omitting all clones

  for(map< float, ArtePointer<RTRA> >::iterator it=mrtra.begin();
      it != mrtra.end(); ++it) {
    int OK = 1;
    for(unsigned int i=0; i!=rtra.size(); ++i) {
      if(rtra[i]->csvd  == it->second->csvd ) OK=0;
      if(rtra[i]->cmain == it->second->cmain) OK=0;
      if(!OK) break;
    }
    if(OK)  rtra.push_back(it->second);
  }
  
  return rtra.size();
} 
