
// *****************************************************************************
//
// source:    rcSelectVZERO2.C
//
// type:      source code
//
// created:   19 Jul 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcSelectVZERO2
//
// called by: rcClassVZERO2
//
// purpose:   select events with V0 and secondary candidates (Denis)
//
// hints:     rc = 0x0:  nothing
//                 0x1:  Ks
//                 0x2:  K892
//                 0x4:  Phi
//                 0x8:  D0
//                 0x10: Ds
//                 0x20: B
//                 0x40: Bs
//
// changes:
// 19 Jul 2000 (AG) creation
// 21 Jul 2000 (AG) included code from Ignacio Arinyo (K0sSelect.C)
// 25 Jul 2000 (AG) corrected bug
// 26 Jul 2000 (AG) included Denis' selection
// 27 Jul 2000 (AG) added D0()
// 05 Aug 2000 (AG) removed printout
// 06 Aug 2000 (AG) added selectB();
// 07 Aug 2000 (AG) selectB() moved to rcSelectDVTX.C
// 06 Sep 2000 (AG) move secondaries to DVTX
//
// *****************************************************************************

// standard includes
#include <iostream.h>

// ARTE include
#include "arte/arte.hh"
#include "farm/farm.hh"

// externals
extern int FADEB;

// prototypes
int selectKs();
int selectK892();
int selectPhi();
int selectD0();
int selectDs();
int selectB();
int selectBs();

//
// function code
//
int
rcSelectVZERO2()
{
  ARTE_mark( "rcSelectVZERO2 0", 0 );
  
  //
  // primaries
  //
  int ks   = selectKs();
  int k892 = selectK892();
  int phi  = selectPhi();
  int d0   = selectD0();
  int ds   = selectDs();
  int b    = selectB();
  int bs   = selectBs();

  // print
  if( FADEB ){
    cout << " SELECT: VZERO2 " 
         << "ks="   << ks   << " "
         << "k892=" << k892 << " "
         << "phi="  << phi  << " "
         << "d0="   << d0   << " "
         << "ds="   << ds   << " "
	 << "b="    << b  << " "
         << "bs="   << bs << " "
         << endl;
   }
  
  //
  // final return code
  //
  int rc = 0x0;

  // VZERO2
  if( ks   ) rc |= 0x1;  // Ks
  if( k892 ) rc |= 0x2;  // K892
  if( phi  ) rc |= 0x4;  // Phi

  // secondaries
  if( d0   ) rc |= 0x8;  // D0
  if( ds   ) rc |= 0x10; // Ds
  if( b    ) rc |= 0x20; // B
  if( bs   ) rc |= 0x40; // Bs

  // print
  if( FADEB ) cout << " SELECT: VZERO2=0x" << hex << rc << dec << endl; 

  ARTE_mark( "rcSelectVZERO2 1", 0 );
  return rc;
}
