#include "rich/RunUtility.h"

MirrorCheck::MirrorCheck()  {;}
MirrorCheck::~MirrorCheck() {;}

int MirrorCheck::calcFromID(int id) {

  RICHDetector &rd=*RICH_Detector;
  LambdaPhi    &lp=*RICH_LambdaPhi;

  if (!rd.ChannelExist(id)) return check::noChannel;
  int   ci = rd.ChannelIndex();
  float tx = tan(lp.phi(ci));
  float ty = tan(lp.lambda(ci)/cos(lp.phi(ci)));
  Vec3  n(tx,ty,1); 
  n=normalize(n);
  Vec3 x0; x0=Vec3(0,0,0);
  StraightLine p(x0,n);
  return calc(p);
}


int MirrorCheck::calcFromTrack(float m[3]) {
  Vec3 x0; x0=Vec3(0,0,0);
  Vec3  n;  n=normalize(Vec3(m[0],m[1],m[2]));
  StraightLine p(x0,n);
  return calc(p);
}



int MirrorCheck::calc(StraightLine p) {
  RICHVessel   &rv=*RICH_Vessel;
  ssm=0;           spm=0;
  xsm=Vec3(0,0,0); xpm=xsm;

  Vec3 coc;
  // SM reflexion
  if (p.n().y()>0) {
    coc=rv.spherical_mirror->UpperMirror().CoC();
    p.x()=p.x()+p.n()*rv.spherical_mirror->UpperMirror().radius();
  }
  else {
    coc=rv.spherical_mirror->LowerMirror().CoC();
    p.x()=p.x()+p.n()*rv.spherical_mirror->LowerMirror().radius();
  }
  Vec3 ns=normalize(p.x()-coc);
  p.n()-=2.*(ns*p.n())*ns;
  if (!(ssm=rv.spherical_mirror->GetSingleMirror(p.x()))) return check::noSM;
  xsm=p.x();
  // PM intersection
  double path;
  if (rv.plane_mirror->fullmirror_intersection(p,&path)) return check::noPM;
  Vec3 x=p.x()+path*p.n();
  if (!(spm=rv.plane_mirror->GetSingleMirror(x))) return check::noPM;
  xpm=x;
  return check::ok;
}
