#ifndef __PM_H__
#define __PM_H__

// ---------------------------------------------------------------------------
///  PM specific information
/**
 The class PM describe a pmt. It stores the type information of the pmt
 and a gain variation factor, which can be different for each channel of
 the pmt.     
 */

class PM {

protected:
  ArtePointer<CPMD> a;
  LensSystem* ls;
  PMType* pmtype;
  char seno[17];
  inline PM() { 
    CPMD d;
    a=ArteTable<CPMD>::push_back(d);
    a->efv0=-1;
    pmtype=0;
    strcpy(seno,"undefined");
    ls=0;
    };
  
public:
/// detection of a photon, which hits the pmt-channel number k.  
  int dodetect(int k,const Photon& p);
/**
 The function "dodetect" describes the detection of a photon with a pmt. 
 It determine which channelis hit. Using the channel dependent gain variation 
 factor and the wavelength dependent quantum efficiency, which is assumed 
 to be the same for pmt's of the same type, the total detection probability
 is calculated. Using a random generator the decision is taken, if the photon
 is detected.
 The return value indicated, if the photon is detected (return value=1)
 or not (return value=0). In the first case the channelid is return
 with the last argument.
 */
 
/// returns a reference to the pmt-type information
  inline const PMType&  type() const  {  return *pmtype; }
/// returns a pointer to the LensSystem description
  inline  LensSystem*  Collector() const  {  return ls; }

/// returns the serialno of the pmt
  inline char*  serialno() {  return seno; }

/// returns the index of the supermodule index, where the pmt is plugged into
  inline int    SuperModule() const { return a->isup; }

/// returns the row, where the pmt is plugged in.
  inline int    Row() const { return a->irow; }

/// returns the column, where the pmt is plugged in
  inline int    Col() const { return a->icol; }

/// return the channel dependend efficiency (gain) variation factor
  inline double efficiency(int i) const { 
    return ArtePointer<CEFF> (a->efv0+i)->eff;
    }
/// set the channel dependend efficiency (gain) variation factor
  float& efficiency(int i) ;

/// shortcut for type().row(i)   (see class PMType for an explanation) 
  inline int   PMrow(int i) const  { return pmtype->row(i); }

/// shortcut for type().col(i)   (see class PMType for an explanation) 
  inline int   PMcol(int i) const  { return pmtype->col(i); }

/// shortcut for type().numchannel() (see class PMType for an explanation)
  inline int   numchannel()    const  { return pmtype->numchannel(); }

/// shortcut for type().numcol() (see class PMType for an explanation) 
  inline int   numcol()        const  { return pmtype->numcol(); }

/// shortcut for type().numrow() (see class PMType for an explanation) 
  inline int   numrol()        const  { return pmtype->numrow(); }

/// shortcut for type().index(i,j) (see class PMType for an explanation) 
  inline int   channelindex(int i,   int j) const  { 
    return pmtype->index(i,j); }

/// shortcut for type().index(x,y) (see class PMType for an explanation) 
  inline int   channelindex(double x,double y) const { 
    return pmtype->index(x,y); }          

///  print the contents of the class to an ASCII-File. See also page: 
  void print(FILE *fp=stdout);

/// constructor reading information from an ASCII-File produced using 'print'
/** constructor reading information from an ASCII-File produced using 'print'. 
 To set the PMT-Type information a List containg all available PMT-Types 
 have to be supplied in the second argument.
  */  
  PM(ScriptFile& f,List<PMType>& PMTypeList,List<LensSystem>& LensSystemList);

/// constructor using a record in the CPMD-ARTE-Table. See also page:
/** constructor using a record in the CPMD-ARTE-Table.
 To set the PMT-Type information a List containg all available PMT-Types 
 have to be supplied in the second argument.
  */  
  PM(ArtePointer<CPMD> a,List<PMType>& PMTypeList,List<LensSystem>& LensSystemList);

/// return the index in table CEFF (special purpose)
  inline int CEFFindex(int i) const { 
    return a->efv0+i;
    }    

  };

#endif
