// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   12. Sept 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: class for a mass constraint
//
// changes:
// 13 Sept 2000 (TG) creation
//
// *****************************************************************************

#include "vt++/VtMassC.hh"
#include "vt++/VtSymMatrix.hh"
#include "vt++/VtRelation.hh"
#include "vt++/VtKalman.hh"

using namespace MATRIX;

namespace VERTEX {

  //==============================================================================
  // Constructors
  //==============================================================================
  MassC::MassC():
    m_mass(0.)
  {}

  MassC::MassC(double m):
    m_mass(m)
  {}

  MassC::~MassC() {
    for(Rel_cit it = m_rel.begin(); it != m_rel.end(); ++it)
      (*it)->massc = 0;
  }

  //==============================================================================
  // push_back
  //==============================================================================
  bool MassC::push_back(Relation* r) {
    if(r->massc != 0) {
#ifdef VtDEBUG
      cout << "MassC: Relation has already a mass constraint assigned!"
	   << endl;
#endif
      return false;
    }

    m_rel.push_back(r);
    r->massc = this;
    return true;
  }

} // end of namespace VERTEX
