#ifndef __OTR_DB_H__
#define __OTR_DB_H__

#ifdef __cplusplus
#include "ledaC++.hh"
extern "C" 
{
#include "ledaC.h"
#else
#include "ledaC.h"
#endif

/**********************************************************
 Definition of OTR Leda database structure.

 Tables:
   OTR_nede        - copy of NEDE tables
   OTR_nesl        - copy of NESL tables
   OTR_cab         - cabling
   OTR_calib       - calibration data (with relation to OTR_r_t)
   OTR_r_t         - space-time relations
   OTR_sharc       - SHARC and FED No.
   OTR_align       - Alignment
   OTR_align_vers  - Version of alignment tables


 AL 11.08.99
**********************************************************/

/********************************************************** NEDE table */

DefineLedaTable (OTR_nede, 1920,
		 DBint inede;
		 DBfloat rot[9];
		 DBfloat xfw[4];
		 DBfloat zfw[4];
		 DBint nwi[4];
		 DBfloat y1w[4];
		 DBfloat y2w[4];
		 DBfloat xmin;
		 DBfloat xmax;
		 DBfloat ymin;
		 DBfloat ymax;
		 DBfloat zmin;
		 DBfloat zmax;
		 DBfloat sxmn;
		 DBfloat sxmx;
		 DBfloat symx;
		 DBfloat symn;
		 DBfloat acc;
		 DBfloat eff;
		 DBfloat dod;
		 DBfloat pit;
		 DBfloat ms;
		 DBfloat cos;
		 DBfloat sin;
		 DBint nsl;
		 DBint cmp;
		 DBint lay;
		 DBint sec;
		 DBint shp;
		 DBint nesl;
		 DBfloat dedx;
		 DBascii hwc[13];
		 DBint gcop;
		 DBascii gmat[5];
		 DBascii gvol[5];
		 DBfloat xmad;
		 DBfloat ymad;
		 DBfloat zmad;
		 DBrelation tonesl; 
		 DBrelation tor_t;  
		 DBrelation toalign;);

/* The difference with NEDE table in ARTE is following:

   Items which are currently not filled: gmat gvol xmad ymad zmad
   Items which are currently not included: dig1 hitl miml fwl nwl
 */

/* Key setting */
// SetLedaKey(1) (nede, DBint, ia);
   SetLedaKey(1) (OTR_nede, DBascii, hwc);

/* Relation setting */
SetLedaRelation(3) (OTR_nede,
                    tor_t, OTR_r_t, tonede,
                    tonesl, OTR_nesl, tonede,
                    toalign, OTR_align, tonede);
 
DefineLedaRelationTable (OTR_nede,
                         OTR_r_t, OTR_r_t_R_nede,
                         OTR_nesl, OTR_nesl_R_nede,
                         OTR_align, OTR_align_R_nede);
 
/*********************************************************** NESL table */
DefineLedaTable (OTR_nesl, 15,
//		 DBint  ib; /* Key field */
		 DBint inesl;
		 DBint cmp;
		 DBint shp;
		 DBint nsl;
		 DBint mfl;
		 DBint lay;
		 DBfloat x1;
		 DBfloat x2;
		 DBfloat y1;
		 DBfloat y2;
		 DBfloat z1;
		 DBfloat z2;
		 DBfloat dx;
		 DBfloat dy;
		 DBint npl;
		 DBint fpl;
		 DBint nwl;
		 DBint fwl;
		 DBint ltyp;
		 DBint pnede;
		 DBascii name[5];
		 DBrelation tonede; /* relation to NEDE */
		 );


/* Key setting */
//SetLedaKey(1) (OTR_nesl, DBint, ib);
SetLedaKey(1) (OTR_nesl, DBascii, name);

/* Relation setting */
SetLedaRelation(1) (OTR_nesl,
                    tonede, OTR_nede, tonesl);
 
DefineLedaRelationTable (OTR_nesl,
                         OTR_nede, OTR_nesl_R_nede);

/******************************************************** Cabling table */
DefineLedaTable (OTR_cab, 6500,
		 DBint  icab; /* Key field */
		 DBascii gencha1[16];
		 DBascii gencha2[16];
		 DBascii modcha1[16];
		 DBascii modcha2[16];
		 DBascii modhv[16];  
		 DBascii ftbpos[16]; 
		 DBascii ftbasd[16]; 
		 DBascii ftbhv[16];  
		 DBascii ftbcab[16]; 
		 DBascii lvpos[16];  
		 DBascii lvcab[16];  
		 DBascii tdccab[16]; 
		 DBascii tdccon[16]; 
		 DBascii previous1[16];
		 DBascii previous2[16];
		 DBascii next1[16];   
		 DBascii next2[16];   
                 DBrelation tosharc; /* relation to Sharc */
                 DBrelation tocalib; /* relation to Calibration */
		 );


/* Key setting */
SetLedaKey(1) (OTR_cab, DBint, icab);

SetLedaRelation(2) (OTR_cab,
                    tosharc, OTR_sharc, tocab,
                    tocalib, OTR_calib, tocab);

DefineLedaRelationTable (OTR_cab,
                         OTR_sharc, OTR_sharc_R_cab,
                         OTR_calib, OTR_calib_R_cab);
 
/******************************************************** Calibration table */
DefineLedaTable (OTR_calib, 6500,
		 DBint  icalib; /* Key field */
		 DBubyte status[16];  /* bits: 0 - dead, cut
					       1 - hot, noisy
					       2 - bad TDC spectrum
					       3 - no HV
					       ...                   */
		 DBubyte t0[16];
		 DBubyte ddt[16];
                 DBrelation tocab; /* relation to Cabling */
		 );


/* Key setting */
SetLedaKey(1) (OTR_calib, DBint, icalib);

/* Relations to Cabling */
SetLedaRelation(1) (OTR_calib,
                    tocab, OTR_cab, tocalib);

DefineLedaRelationTable (OTR_calib,
                         OTR_cab, OTR_calib_R_cab);

/******************************************************** r(t) table */
DefineLedaTable (OTR_r_t, 6500,
		 DBint  irt; /* Key field */
		 DBubyte meth ; /* Method of parameterization */
		 DBubyte pitch;   /* 0=5mm, 1=10mm */
		 DBuint n_rt;
		 DBfloat par_rt[20];
		 DBuint n_res;
		 DBfloat par_res[20];
                 DBrelation tonede; /* relation to NEDE */
		 );


/* Key setting */
SetLedaKey(1) (OTR_r_t, DBint, irt);

/* Relations to NEDE */
SetLedaRelation(1) (OTR_r_t,
                    tonede, OTR_nede, tor_t);

DefineLedaRelationTable (OTR_r_t,
                         OTR_nede, OTR_r_t_R_nede);

/******************************************************** Sharc table */
DefineLedaTable (OTR_sharc, 200,
		 DBint  isharc; /* Key field */
		 DBascii board[9];
		 DBint fed;
		 DBascii sharc_cable[12];
		 DBint sharc_board;
		 DBint sharc_board_pos;
                 DBrelation tocab; /* relation container */
		 );


/* Key setting */
SetLedaKey(1) (OTR_sharc, DBint, isharc);

/* Relation setting */
SetLedaRelation(1) (OTR_sharc,
                    tocab, OTR_cab, tosharc);

DefineLedaRelationTable (OTR_sharc,
                         OTR_cab, OTR_sharc_R_cab);


/******************************************************** Alignment */

/* OTR_ALIGN */
/* Alignment Constants for each OTR_Chamber */
/* 05/08/99 TZ */
/* 06/08/99 DK */
/* 10/08/99 AL */

DefineLedaTable (OTR_align, 1920,
                 DBushort	 	id;          
                 DBrelation             tonede;      /* Relation to corresponding NEDE */
                 DBfloat                ang[3];      /* three rotations */ 
                 DBfloat                eang[3];     /* error for rotations */
                 DBfloat                delt[3];     /* three translations */
                 DBfloat                edelt[3];    /* error for translations */

                 /* OTR specific stuff */
                 DBfloat                sloff[3];    /* offset of sublayer (=monolayer) 
						        within module */
                 DBfloat       		esloff[3];   /* error of sublayer offset */
                 DBfloat                redres;      /* width of residual distribution 
							extrapolated to a track momentum
                                                        of infinity */
                 );

SetLedaKey(1) (OTR_align, DBushort, id);

SetLedaRelation(1) (OTR_align,
		    tonede, OTR_nede, toalign);

DefineLedaRelationTable (OTR_align, 
                         OTR_nede,      OTR_align_R_nede);

/******************************************************** Alignment version */

DefineLedaTable (OTR_align_vers, 3,
                 DBint			vers;        /* version of  alignment program */
                 DBint			run1;        /* first, last run used for */
		 DBint			runl;        /*   alignment */
                 DBfloat		qual[10];    /* quality of alignment */
                 DBascii                dqual[20];   /* description of quality */
                 );                                  /* I put length 20. AL */

SetLedaKey(1) (OTR_align_vers, DBint, run1);

#ifdef __cplusplus
}
#endif

#endif
