/**********************************************************
 Routine for creating OTGL and OTMO tables.

 Should be called during initialization,
 but after GEDE tables are defined.
 
 A.Lanyov    September 1999
 **********************************************************/
 
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include <arte/arte.hh>
#include <arte/artefunc.h>
 
extern "C" {
int ot_init(){
  ArteTable<OTGL>::clear();
  ArteTable<OTMO>::clear();

  //************** Book OTGL table
  ArtePointer<OTGL> otgl = ArteTable<OTGL>::push_back( OTGL() );
  //  ArtePointer<OTGL> otgl(1); 
  
  int ndts = 1;
  for ( ArteTableIterator<GEDE> gedeIt; gedeIt.valid(); gedeIt.next() ) {
    
    if (gedeIt->cmp != 2 || gedeIt->sec == 0) {
      continue;
    };

    //************** Book new OTMO table and fill it with (some) data
    ArtePointer<OTMO> otmo = ArteTable<OTMO>::push_back( OTMO() );
    //ArtePointer<OTMO> otmo(ArtePointer<OTMO>::nrow()); 

    // mean y-coordinate for what t_zero was defined
    otmo->yt0 = 0.5*(gedeIt->ymin + gedeIt->ymax); 

    // direction of signal propagation on wire
    otmo->ydir = int(gedeIt->zmad);

    // first word in table OTTZ(OTTE) related to module
    otmo->idtz = ndts;
    ndts += (3 + gedeIt->nwi[0]
	       + gedeIt->nwi[1]
	       + gedeIt->nwi[2]
	       + gedeIt->nwi[3]) / 4;

    /////identifier of time-space relations stored in OTTS
    //    if (abs(gedeIt->pit - 0.87) < 0.1) {
    //      otmo->idts = 1;   // 5-mm modules
    //    }
    //    else {
    //      otmo->idts = 2;   // 10-mm modules
    //    };
    otmo->idts = 0;   // will be initialized in another routine?
    //    otmo->idts = 
    //   waxRTManager::GetMe()->GetRTIndexFromGedeindex(gedeIt.index()) ;
    
    (*gedeIt)->otmo = otmo;
    otmo->gede      = *gedeIt;
    
  };
  
  printf("* ot_init: %d OTMO tables were booked.         *\n", ArteTable<OTMO>::size());

  //*************** Fill OTGL table it with (some) data
  
  otgl->gede = ArtePointer<OTMO>::nrow();   // number of rows in gede related to ot(=2485)
  otgl->mchn = 128;            // maximal number of channel(wire)/gede row (=128)
//otgl->medb;                  // method of storage of time-space relations in db
//otgl->meth;                  // method of storage of time-space relations in otts
//otgl->knot;                  // number of knots for tabulation(=129)
  otgl->wvel = 28.2 ;          // signal propagation velocity on wire (=28.2 cm/ns)
  otgl->vd[0] = 0.01;          // [0] average drift velocity for 5mm  (=0.01 cm/ns)
  otgl->vd[1] = 0.01;          // [1] average drift velocity for 5mm  (=0.01 cm/ns)
  otgl->vdmf[0] = 0.01;        // [0]                            5mm  in magnet
  otgl->vdmf[1] = 0.01;        // [1]                            5mm  in magnet
  otgl->fele = 0.8;            // electronic resolution in ns (=0.8)
  otgl->ntdc = 0.39;           // ns per one tdc count (=0.39 ns/tdc count)
  otgl->fatt = 0.25;           // fraction of electrons that reach the wire (=0.25)
  otgl->damp = 0.2;            //             charge seen by the amplifier (=0.2)
  otgl->db   = 1.222;          // parameter of polya distribution (=1.222)
  otgl->dqe  = 1.602e-19;      // elementary charge in Coulomb (=1.602e-19)
  otgl->dlam = 102.;           // doubled primary ionization density (=102. 1/cm)
//otgl->xtcl;                  // probability of x-talk through cell
//otgl->xtre;                  //                               readout circuit

  printf("* ot_init: %d OTGL table was booked.              *\n", ArteTable<OTGL>::size());

  return 0;
}
}
